/* Generated by Cython 0.21.1 */

#define PY_SSIZE_T_CLEAN
#ifndef CYTHON_USE_PYLONG_INTERNALS
#ifdef PYLONG_BITS_IN_DIGIT
#define CYTHON_USE_PYLONG_INTERNALS 0
#else
#include "pyconfig.h"
#ifdef PYLONG_BITS_IN_DIGIT
#define CYTHON_USE_PYLONG_INTERNALS 1
#else
#define CYTHON_USE_PYLONG_INTERNALS 0
#endif
#endif
#endif
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03020000)
    #error Cython requires Python 2.6+ or Python 3.2+.
#else
#define CYTHON_ABI "0_21_1"
#include <stddef.h>
#ifndef offsetof
#define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
#define CYTHON_COMPILING_IN_PYPY 1
#define CYTHON_COMPILING_IN_CPYTHON 0
#else
#define CYTHON_COMPILING_IN_PYPY 0
#define CYTHON_COMPILING_IN_CPYTHON 1
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600
#define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos) \
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos) \
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#if PY_MAJOR_VERSION >= 3
  #define Py_TPFLAGS_CHECKTYPES 0
  #define Py_TPFLAGS_HAVE_INDEX 0
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#if PY_VERSION_HEX < 0x030400a1 && !defined(Py_TPFLAGS_HAVE_FINALIZE)
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ? \
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
#else
  #define CYTHON_PEP393_ENABLED 0
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
  #define __Pyx_PyFrozenSet_Size(s)         PyObject_Size(s)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ? \
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
  #define __Pyx_PyFrozenSet_Size(s)         PySet_Size(s)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#ifndef CYTHON_INLINE
  #if defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  /* Initialize NaN. The sign is irrelevant, an exponent with all bits 1 and
   a nonzero mantissa means NaN. If the first bit in the mantissa is 1, it is
   a quiet NaN. */
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#ifdef __cplusplus
template<typename T>
void __Pyx_call_destructor(T* x) {
    x->~T();
}
#endif


#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#endif

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
#define _USE_MATH_DEFINES
#endif
#include <math.h>
#define __PYX_HAVE___gldraw
#define __PYX_HAVE_API___gldraw
#include "stddef.h"
#include "stdint.h"
#include "GL/gl.h"
#include "GL/glext.h"
#include "math.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#ifdef PYREX_WITHOUT_ASSERTIONS
#define CYTHON_WITHOUT_ASSERTIONS
#endif

#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
typedef struct {PyObject **p; char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (    \
    (sizeof(type) < sizeof(Py_ssize_t))  ||             \
    (sizeof(type) > sizeof(Py_ssize_t) &&               \
          likely(v < (type)PY_SSIZE_T_MAX ||            \
                 v == (type)PY_SSIZE_T_MAX)  &&         \
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||       \
                                v == (type)PY_SSIZE_T_MIN)))  ||  \
    (sizeof(type) == sizeof(Py_ssize_t) &&              \
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||        \
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyObject_AsSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromUString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromUString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromUString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromUString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromUString(s) __Pyx_PyUnicode_FromString((const char*)s)
#if PY_MAJOR_VERSION < 3
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u)
{
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#else
#define __Pyx_Py_UNICODE_strlen Py_UNICODE_strlen
#endif
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_Owned_Py_None(b) (Py_INCREF(Py_None), Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? (Py_INCREF(Py_True), Py_True) : (Py_INCREF(Py_False), Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Int(PyObject* x);
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_COMPILING_IN_CPYTHON
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */

static PyObject *__pyx_m;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "_gldraw.pyx",
};

/* "gl.pxd":64
 * # from /usr/include/GL/gl.h:
 * 
 * ctypedef unsigned int	GLenum             # <<<<<<<<<<<<<<
 * ctypedef unsigned char	GLboolean
 * ctypedef unsigned int	GLbitfield
 */
typedef unsigned int __pyx_t_2gl_GLenum;

/* "gl.pxd":65
 * 
 * ctypedef unsigned int	GLenum
 * ctypedef unsigned char	GLboolean             # <<<<<<<<<<<<<<
 * ctypedef unsigned int	GLbitfield
 * ctypedef void		GLvoid
 */
typedef unsigned char __pyx_t_2gl_GLboolean;

/* "gl.pxd":66
 * ctypedef unsigned int	GLenum
 * ctypedef unsigned char	GLboolean
 * ctypedef unsigned int	GLbitfield             # <<<<<<<<<<<<<<
 * ctypedef void		GLvoid
 * ctypedef int		GLint
 */
typedef unsigned int __pyx_t_2gl_GLbitfield;

/* "gl.pxd":68
 * ctypedef unsigned int	GLbitfield
 * ctypedef void		GLvoid
 * ctypedef int		GLint             # <<<<<<<<<<<<<<
 * ctypedef unsigned char	GLubyte
 * ctypedef unsigned int	GLuint
 */
typedef int __pyx_t_2gl_GLint;

/* "gl.pxd":69
 * ctypedef void		GLvoid
 * ctypedef int		GLint
 * ctypedef unsigned char	GLubyte             # <<<<<<<<<<<<<<
 * ctypedef unsigned int	GLuint
 * ctypedef int		GLsizei
 */
typedef unsigned char __pyx_t_2gl_GLubyte;

/* "gl.pxd":70
 * ctypedef int		GLint
 * ctypedef unsigned char	GLubyte
 * ctypedef unsigned int	GLuint             # <<<<<<<<<<<<<<
 * ctypedef int		GLsizei
 * ctypedef float		GLfloat
 */
typedef unsigned int __pyx_t_2gl_GLuint;

/* "gl.pxd":71
 * ctypedef unsigned char	GLubyte
 * ctypedef unsigned int	GLuint
 * ctypedef int		GLsizei             # <<<<<<<<<<<<<<
 * ctypedef float		GLfloat
 * ctypedef float		GLclampf
 */
typedef int __pyx_t_2gl_GLsizei;

/* "gl.pxd":72
 * ctypedef unsigned int	GLuint
 * ctypedef int		GLsizei
 * ctypedef float		GLfloat             # <<<<<<<<<<<<<<
 * ctypedef float		GLclampf
 * 
 */
typedef float __pyx_t_2gl_GLfloat;

/* "gl.pxd":73
 * ctypedef int		GLsizei
 * ctypedef float		GLfloat
 * ctypedef float		GLclampf             # <<<<<<<<<<<<<<
 * 
 * 
 */
typedef float __pyx_t_2gl_GLclampf;

/* "gl.pxd":78
 * # from /usr/include/GL/glext.h:
 * 
 * ctypedef ptrdiff_t GLsizeiptr             # <<<<<<<<<<<<<<
 * ctypedef ptrdiff_t GLintptr
 * ctypedef char GLchar
 */
typedef ptrdiff_t __pyx_t_2gl_GLsizeiptr;

/* "gl.pxd":79
 * 
 * ctypedef ptrdiff_t GLsizeiptr
 * ctypedef ptrdiff_t GLintptr             # <<<<<<<<<<<<<<
 * ctypedef char GLchar
 * 
 */
typedef ptrdiff_t __pyx_t_2gl_GLintptr;

/* "gl.pxd":80
 * ctypedef ptrdiff_t GLsizeiptr
 * ctypedef ptrdiff_t GLintptr
 * ctypedef char GLchar             # <<<<<<<<<<<<<<
 * 
 * 
 */
typedef char __pyx_t_2gl_GLchar;

/*--- Type declarations ---*/

/* "gl.pxd":67
 * ctypedef unsigned char	GLboolean
 * ctypedef unsigned int	GLbitfield
 * ctypedef void		GLvoid             # <<<<<<<<<<<<<<
 * ctypedef int		GLint
 * ctypedef unsigned char	GLubyte
 */
typedef void __pyx_t_2gl_GLvoid;
struct __pyx_t_7_gldraw_Block;
struct __pyx_t_7_gldraw_Cube;
struct __pyx_t_7_gldraw_Animation_Struct;

/* "_gldraw.pxd":4
 * 
 * from gl cimport *    #line 35
 * cdef enum: #    #line 52             # <<<<<<<<<<<<<<
 *     ATTRIB_LOCATION = 0    #line 57
 *     PICKATTRIB_LOCATION = 5    #line 58
 */
enum  {
  __pyx_e_7_gldraw_ATTRIB_LOCATION = 0,
  __pyx_e_7_gldraw_PICKATTRIB_LOCATION = 5
};

/* "_gldraw.pxd":7
 *     ATTRIB_LOCATION = 0    #line 57
 *     PICKATTRIB_LOCATION = 5    #line 58
 * ctypedef float vec4[4]    #line 61             # <<<<<<<<<<<<<<
 * ctypedef vec4 mat4[4]    #line 62
 * cdef matrix_set_identity(mat4& matrix)    #line 66
 */
typedef float __pyx_t_7_gldraw_vec4[4];

/* "_gldraw.pxd":8
 *     PICKATTRIB_LOCATION = 5    #line 58
 * ctypedef float vec4[4]    #line 61
 * ctypedef vec4 mat4[4]    #line 62             # <<<<<<<<<<<<<<
 * cdef matrix_set_identity(mat4& matrix)    #line 66
 * cdef void gl_draw_cube()    #line 186
 */
typedef __pyx_t_7_gldraw_vec4 __pyx_t_7_gldraw_mat4[4];

/* "_gldraw.pyx":52
 * 
 * 
 * cdef enum: # #pxd/             # <<<<<<<<<<<<<<
 * #if True:
 *     MAX_TRANSFORMATIONS = 24
 */
enum  {
  __pyx_e_7_gldraw_MAX_TRANSFORMATIONS = 24,
  __pyx_e_7_gldraw_MAX_BLOCKS = 1000
};

/* "_gldraw.pyx":73
 *     matrix[3][0] = 0.; matrix[3][1] = 0.; matrix[3][2] = 0.; matrix[3][3] = 1.
 * 
 * cdef struct Block: #px/             # <<<<<<<<<<<<<<
 * #class Block:     # pylint: disable=R0903
 * ##px-
 */
struct __pyx_t_7_gldraw_Block {
  __pyx_t_7_gldraw_vec4 *transformation;
  int in_motion;
  int idx_triangles;
  int cnt_triangles;
};

/* "_gldraw.pyx":88
 *         #self.cnt_triangles = None
 * 
 * cdef struct Cube: #px/             # <<<<<<<<<<<<<<
 * #class cube:     # pylint: disable=W0232, R0903
 *     mat4 transformations[MAX_TRANSFORMATIONS] #px/
 */
struct __pyx_t_7_gldraw_Cube {
  __pyx_t_7_gldraw_mat4 transformations[__pyx_e_7_gldraw_MAX_TRANSFORMATIONS];
  unsigned int number_blocks;
  struct __pyx_t_7_gldraw_Block blocks[__pyx_e_7_gldraw_MAX_BLOCKS];
  int cnt_pick;
  int idx_debug;
  __pyx_t_2gl_GLuint object_location;
  __pyx_t_2gl_GLuint glbuffer;
};

/* "_gldraw.pyx":105
 * cdef Cube cube #px+
 * 
 * cdef struct Animation_Struct: #px/             # <<<<<<<<<<<<<<
 * #class animation:
 *     float angle, angle_max #px+
 */
struct __pyx_t_7_gldraw_Animation_Struct {
  float angle;
  float angle_max;
  float rotation_x;
  float rotation_y;
  float rotation_z;
  __pyx_t_7_gldraw_mat4 rotation_matrix;
};
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil) \
          if (acquire_gil) { \
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure(); \
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__); \
              PyGILState_Release(__pyx_gilstate_save); \
          } else { \
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__); \
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil) \
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext() \
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {                            \
        PyObject *tmp = (PyObject *) r;                         \
        r = v; __Pyx_XDECREF(tmp);                              \
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {                             \
        PyObject *tmp = (PyObject *) r;                         \
        r = v; __Pyx_DECREF(tmp);                               \
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

static PyObject *__Pyx_GetBuiltinName(PyObject *name);

static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[], \
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args, \
    const char* function_name);

static CYTHON_INLINE int __Pyx_ArgTypeTest(PyObject *obj, PyTypeObject *type, int none_allowed,
    const char *name, int exact);

static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

static CYTHON_INLINE int __Pyx_IterFinish(void);

static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck) \
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ? \
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) : \
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) : \
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck) \
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ? \
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) : \
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck) \
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ? \
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) : \
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

static double __Pyx__PyObject_AsDouble(PyObject* obj);
#if CYTHON_COMPILING_IN_PYPY
#define __Pyx_PyObject_AsDouble(obj) \
(likely(PyFloat_CheckExact(obj)) ? PyFloat_AS_DOUBLE(obj) : \
 likely(PyInt_CheckExact(obj)) ? \
 PyFloat_AsDouble(obj) : __Pyx__PyObject_AsDouble(obj))
#else
#define __Pyx_PyObject_AsDouble(obj) \
((likely(PyFloat_CheckExact(obj))) ? \
 PyFloat_AS_DOUBLE(obj) : __Pyx__PyObject_AsDouble(obj))
#endif

static CYTHON_INLINE void __Pyx_ErrRestore(PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetch(PyObject **type, PyObject **value, PyObject **tb);

static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback);

static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name);

typedef struct {
    int code_line;
    PyCodeObject* code_object;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

static CYTHON_INLINE unsigned int __Pyx_PyInt_As_unsigned_int(PyObject *);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_int(unsigned int value);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

static int __Pyx_check_binary_version(void);

static int __Pyx_ExportFunction(const char *name, void (*f)(void), const char *sig);

static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);


/* Module declarations from 'libc.stddef' */

/* Module declarations from 'libc.stdint' */

/* Module declarations from 'gl' */

/* Module declarations from 'libc.math' */

/* Module declarations from '_gldraw' */
static struct __pyx_t_7_gldraw_Cube __pyx_v_7_gldraw_cube;
static struct __pyx_t_7_gldraw_Animation_Struct __pyx_v_7_gldraw_animation;
static PyObject *__pyx_f_7_gldraw_matrix_set_identity(__pyx_t_7_gldraw_vec4 *); /*proto*/
static PyObject *__pyx_f_7_gldraw_gl_set_atlas_texture(int, int, int, PyObject *, int __pyx_skip_dispatch); /*proto*/
static PyObject *__pyx_f_7_gldraw_set_animation_start(PyObject *, float, float, float, float, int __pyx_skip_dispatch); /*proto*/
static void __pyx_f_7_gldraw__update_animation_rotation_matrix(void); /*proto*/
static PyObject *__pyx_f_7_gldraw_set_animation_next(float, int __pyx_skip_dispatch); /*proto*/
static void __pyx_f_7_gldraw__mult_matrix(__pyx_t_7_gldraw_vec4 *, __pyx_t_7_gldraw_vec4 *, __pyx_t_7_gldraw_vec4 *); /*proto*/
static void __pyx_f_7_gldraw__gl_set_pointer(__pyx_t_2gl_GLuint, __pyx_t_2gl_GLint, __pyx_t_2gl_GLenum, __pyx_t_2gl_GLboolean, long); /*proto*/
static PyObject *__pyx_f_7_gldraw_gl_set_data(int, PyObject *, PyObject *, PyObject *, PyObject *, int __pyx_skip_dispatch); /*proto*/
#define __Pyx_MODULE_NAME "_gldraw"
int __pyx_module_is_main__gldraw = 0;

/* Implementation of '_gldraw' */
static PyObject *__pyx_builtin_enumerate;
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_pf_7_gldraw_init_module(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_7_gldraw_2set_transformations(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_blocks); /* proto */
static PyObject *__pyx_pf_7_gldraw_4gl_set_atlas_texture(CYTHON_UNUSED PyObject *__pyx_self, int __pyx_v_x, int __pyx_v_w, int __pyx_v_h, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_7_gldraw_6set_animation_start(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_blocks, float __pyx_v_angle_max, float __pyx_v_axisx, float __pyx_v_axisy, float __pyx_v_axisz); /* proto */
static PyObject *__pyx_pf_7_gldraw_8set_animation_next(CYTHON_UNUSED PyObject *__pyx_self, float __pyx_v_increment); /* proto */
static PyObject *__pyx_pf_7_gldraw_10gl_set_data(CYTHON_UNUSED PyObject *__pyx_self, int __pyx_v_nblocks, PyObject *__pyx_v_vertexdata, PyObject *__pyx_v_vertexpointers, PyObject *__pyx_v_vertexinfo, PyObject *__pyx_v_transformations); /* proto */
static char __pyx_k_b[] = "b";
static char __pyx_k_h[] = "h";
static char __pyx_k_i[] = "i";
static char __pyx_k_w[] = "w";
static char __pyx_k_x[] = "x";
static char __pyx_k_all[] = "__all__";
static char __pyx_k_data[] = "data";
static char __pyx_k_main[] = "__main__";
static char __pyx_k_mat4[] = "mat4";
static char __pyx_k_name[] = "__name__";
static char __pyx_k_test[] = "__test__";
static char __pyx_k_DEBUG[] = "DEBUG";
static char __pyx_k_axisx[] = "axisx";
static char __pyx_k_axisy[] = "axisy";
static char __pyx_k_axisz[] = "axisz";
static char __pyx_k_debug[] = "debug";
static char __pyx_k_range[] = "range";
static char __pyx_k_blocks[] = "blocks";
static char __pyx_k_format[] = "format";
static char __pyx_k_gldraw[] = "_gldraw";
static char __pyx_k_import[] = "__import__";
static char __pyx_k_nblocks[] = "nblocks";
static char __pyx_k_package[] = "__package__";
static char __pyx_k_compiled[] = "__compiled";
static char __pyx_k_angle_max[] = "angle_max";
static char __pyx_k_enumerate[] = "enumerate";
static char __pyx_k_compiled_2[] = "  compiled:";
static char __pyx_k_vertexdata[] = "vertexdata";
static char __pyx_k_vertexinfo[] = "vertexinfo";
static char __pyx_k_init_module[] = "init_module";
static char __pyx_k_from_package[] = "  from package:";
static char __pyx_k_gl_draw_cube[] = "gl_draw_cube";
static char __pyx_k_gl_pick_cube[] = "gl_pick_cube";
static char __pyx_k_vertexpointers[] = "vertexpointers";
static char __pyx_k_ATTRIB_LOCATION[] = "ATTRIB_LOCATION";
static char __pyx_k_gl_init_buffers[] = "gl_init_buffers";
static char __pyx_k_transformations[] = "transformations";
static char __pyx_k_Importing_module[] = "Importing module:";
static char __pyx_k_gl_delete_buffers[] = "gl_delete_buffers";
static char __pyx_k_gl_draw_cube_debug[] = "gl_draw_cube_debug";
static char __pyx_k_PICKATTRIB_LOCATION[] = "PICKATTRIB_LOCATION";
static char __pyx_k_matrix_set_identity[] = "matrix_set_identity";
static char __pyx_k_set_transformations[] = "set_transformations";
static char __pyx_k_gl_draw_select_debug[] = "gl_draw_select_debug";
static char __pyx_k_gl_init_object_location[] = "gl_init_object_location";
static char __pyx_k_blocks_hardcoded_limit_is[] = "{} blocks, hardcoded limit is {}";
static char __pyx_k_tmp[] = "/tmp/build/temp.linux-x86_64-3.4/pybiklib/_gldraw.pyx";
static char __pyx_k_transformations_hardcoded_limit[] = "{} transformations, hardcoded limit is {}";
static PyObject *__pyx_n_u_ATTRIB_LOCATION;
static PyObject *__pyx_n_s_DEBUG;
static PyObject *__pyx_kp_u_Importing_module;
static PyObject *__pyx_n_u_PICKATTRIB_LOCATION;
static PyObject *__pyx_n_s_all;
static PyObject *__pyx_n_s_angle_max;
static PyObject *__pyx_n_s_axisx;
static PyObject *__pyx_n_s_axisy;
static PyObject *__pyx_n_s_axisz;
static PyObject *__pyx_n_s_b;
static PyObject *__pyx_n_s_blocks;
static PyObject *__pyx_kp_u_blocks_hardcoded_limit_is;
static PyObject *__pyx_n_s_compiled;
static PyObject *__pyx_kp_u_compiled_2;
static PyObject *__pyx_n_s_data;
static PyObject *__pyx_n_s_debug;
static PyObject *__pyx_n_s_enumerate;
static PyObject *__pyx_n_s_format;
static PyObject *__pyx_kp_u_from_package;
static PyObject *__pyx_n_u_gl_delete_buffers;
static PyObject *__pyx_n_u_gl_draw_cube;
static PyObject *__pyx_n_u_gl_draw_cube_debug;
static PyObject *__pyx_n_u_gl_draw_select_debug;
static PyObject *__pyx_n_u_gl_init_buffers;
static PyObject *__pyx_n_u_gl_init_object_location;
static PyObject *__pyx_n_u_gl_pick_cube;
static PyObject *__pyx_n_s_gldraw;
static PyObject *__pyx_n_s_h;
static PyObject *__pyx_kp_s_tmp;
static PyObject *__pyx_n_s_i;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_init_module;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_u_mat4;
static PyObject *__pyx_n_u_matrix_set_identity;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_nblocks;
static PyObject *__pyx_n_s_package;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_set_transformations;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_transformations;
static PyObject *__pyx_kp_u_transformations_hardcoded_limit;
static PyObject *__pyx_n_s_vertexdata;
static PyObject *__pyx_n_s_vertexinfo;
static PyObject *__pyx_n_s_vertexpointers;
static PyObject *__pyx_n_s_w;
static PyObject *__pyx_n_s_x;
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_codeobj__2;
static PyObject *__pyx_codeobj__4;

/* "_gldraw.pyx":66
 * #mat4 = lambda: [[0.]*4 for _i in range(4)]
 * 
 * cdef matrix_set_identity(mat4& matrix): #pxd/             # <<<<<<<<<<<<<<
 * #def matrix_set_identity(matrix):
 *     matrix[0][0] = 1.; matrix[0][1] = 0.; matrix[0][2] = 0.; matrix[0][3] = 0.
 */

static PyObject *__pyx_f_7_gldraw_matrix_set_identity(__pyx_t_7_gldraw_vec4 *__pyx_v_matrix) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("matrix_set_identity", 0);

  /* "_gldraw.pyx":68
 * cdef matrix_set_identity(mat4& matrix): #pxd/
 * #def matrix_set_identity(matrix):
 *     matrix[0][0] = 1.; matrix[0][1] = 0.; matrix[0][2] = 0.; matrix[0][3] = 0.             # <<<<<<<<<<<<<<
 *     matrix[1][0] = 0.; matrix[1][1] = 1.; matrix[1][2] = 0.; matrix[1][3] = 0.
 *     matrix[2][0] = 0.; matrix[2][1] = 0.; matrix[2][2] = 1.; matrix[2][3] = 0.
 */
  ((__pyx_v_matrix[0])[0]) = 1.;
  ((__pyx_v_matrix[0])[1]) = 0.;
  ((__pyx_v_matrix[0])[2]) = 0.;
  ((__pyx_v_matrix[0])[3]) = 0.;

  /* "_gldraw.pyx":69
 * #def matrix_set_identity(matrix):
 *     matrix[0][0] = 1.; matrix[0][1] = 0.; matrix[0][2] = 0.; matrix[0][3] = 0.
 *     matrix[1][0] = 0.; matrix[1][1] = 1.; matrix[1][2] = 0.; matrix[1][3] = 0.             # <<<<<<<<<<<<<<
 *     matrix[2][0] = 0.; matrix[2][1] = 0.; matrix[2][2] = 1.; matrix[2][3] = 0.
 *     matrix[3][0] = 0.; matrix[3][1] = 0.; matrix[3][2] = 0.; matrix[3][3] = 1.
 */
  ((__pyx_v_matrix[1])[0]) = 0.;
  ((__pyx_v_matrix[1])[1]) = 1.;
  ((__pyx_v_matrix[1])[2]) = 0.;
  ((__pyx_v_matrix[1])[3]) = 0.;

  /* "_gldraw.pyx":70
 *     matrix[0][0] = 1.; matrix[0][1] = 0.; matrix[0][2] = 0.; matrix[0][3] = 0.
 *     matrix[1][0] = 0.; matrix[1][1] = 1.; matrix[1][2] = 0.; matrix[1][3] = 0.
 *     matrix[2][0] = 0.; matrix[2][1] = 0.; matrix[2][2] = 1.; matrix[2][3] = 0.             # <<<<<<<<<<<<<<
 *     matrix[3][0] = 0.; matrix[3][1] = 0.; matrix[3][2] = 0.; matrix[3][3] = 1.
 * 
 */
  ((__pyx_v_matrix[2])[0]) = 0.;
  ((__pyx_v_matrix[2])[1]) = 0.;
  ((__pyx_v_matrix[2])[2]) = 1.;
  ((__pyx_v_matrix[2])[3]) = 0.;

  /* "_gldraw.pyx":71
 *     matrix[1][0] = 0.; matrix[1][1] = 1.; matrix[1][2] = 0.; matrix[1][3] = 0.
 *     matrix[2][0] = 0.; matrix[2][1] = 0.; matrix[2][2] = 1.; matrix[2][3] = 0.
 *     matrix[3][0] = 0.; matrix[3][1] = 0.; matrix[3][2] = 0.; matrix[3][3] = 1.             # <<<<<<<<<<<<<<
 * 
 * cdef struct Block: #px/
 */
  ((__pyx_v_matrix[3])[0]) = 0.;
  ((__pyx_v_matrix[3])[1]) = 0.;
  ((__pyx_v_matrix[3])[2]) = 0.;
  ((__pyx_v_matrix[3])[3]) = 1.;

  /* "_gldraw.pyx":66
 * #mat4 = lambda: [[0.]*4 for _i in range(4)]
 * 
 * cdef matrix_set_identity(mat4& matrix): #pxd/             # <<<<<<<<<<<<<<
 * #def matrix_set_identity(matrix):
 *     matrix[0][0] = 1.; matrix[0][1] = 0.; matrix[0][2] = 0.; matrix[0][3] = 0.
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_gldraw.pyx":114
 * 
 * 
 * def init_module():             # <<<<<<<<<<<<<<
 *     cdef int i #px+
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7_gldraw_1init_module(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_7_gldraw_1init_module = {"init_module", (PyCFunction)__pyx_pw_7_gldraw_1init_module, METH_NOARGS, 0};
static PyObject *__pyx_pw_7_gldraw_1init_module(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("init_module (wrapper)", 0);
  __pyx_r = __pyx_pf_7_gldraw_init_module(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7_gldraw_init_module(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("init_module", 0);

  /* "_gldraw.pyx":117
 *     cdef int i #px+
 * 
 *     cube.number_blocks = 0             # <<<<<<<<<<<<<<
 *     #cube.blocks
 * 
 */
  __pyx_v_7_gldraw_cube.number_blocks = 0;

  /* "_gldraw.pyx":120
 *     #cube.blocks
 * 
 *     cube.cnt_pick = 0             # <<<<<<<<<<<<<<
 *     cube.idx_debug = 0
 * 
 */
  __pyx_v_7_gldraw_cube.cnt_pick = 0;

  /* "_gldraw.pyx":121
 * 
 *     cube.cnt_pick = 0
 *     cube.idx_debug = 0             # <<<<<<<<<<<<<<
 * 
 *     animation.angle = animation.angle_max = 0
 */
  __pyx_v_7_gldraw_cube.idx_debug = 0;

  /* "_gldraw.pyx":123
 *     cube.idx_debug = 0
 * 
 *     animation.angle = animation.angle_max = 0             # <<<<<<<<<<<<<<
 * 
 * def set_transformations(blocks):
 */
  __pyx_v_7_gldraw_animation.angle = 0.0;
  __pyx_v_7_gldraw_animation.angle_max = 0.0;

  /* "_gldraw.pyx":114
 * 
 * 
 * def init_module():             # <<<<<<<<<<<<<<
 *     cdef int i #px+
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_gldraw.pyx":125
 *     animation.angle = animation.angle_max = 0
 * 
 * def set_transformations(blocks):             # <<<<<<<<<<<<<<
 *     cdef unsigned int i, b #px+
 *     for i, b in enumerate(blocks):
 */

/* Python wrapper */
static PyObject *__pyx_pw_7_gldraw_3set_transformations(PyObject *__pyx_self, PyObject *__pyx_v_blocks); /*proto*/
static PyMethodDef __pyx_mdef_7_gldraw_3set_transformations = {"set_transformations", (PyCFunction)__pyx_pw_7_gldraw_3set_transformations, METH_O, 0};
static PyObject *__pyx_pw_7_gldraw_3set_transformations(PyObject *__pyx_self, PyObject *__pyx_v_blocks) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_transformations (wrapper)", 0);
  __pyx_r = __pyx_pf_7_gldraw_2set_transformations(__pyx_self, ((PyObject *)__pyx_v_blocks));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7_gldraw_2set_transformations(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_blocks) {
  unsigned int __pyx_v_i;
  unsigned int __pyx_v_b;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  unsigned int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *(*__pyx_t_4)(PyObject *);
  PyObject *__pyx_t_5 = NULL;
  unsigned int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_transformations", 0);

  /* "_gldraw.pyx":127
 * def set_transformations(blocks):
 *     cdef unsigned int i, b #px+
 *     for i, b in enumerate(blocks):             # <<<<<<<<<<<<<<
 *         cube.blocks[i].transformation = cube.transformations[b]
 *         cube.blocks[i].in_motion = False
 */
  __pyx_t_1 = 0;
  if (likely(PyList_CheckExact(__pyx_v_blocks)) || PyTuple_CheckExact(__pyx_v_blocks)) {
    __pyx_t_2 = __pyx_v_blocks; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;
    __pyx_t_4 = NULL;
  } else {
    __pyx_t_3 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_v_blocks); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  for (;;) {
    if (likely(!__pyx_t_4)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_5 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_5 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #endif
      } else {
        if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_5 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #endif
      }
    } else {
      __pyx_t_5 = __pyx_t_4(__pyx_t_2);
      if (unlikely(!__pyx_t_5)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_5);
    }
    __pyx_t_6 = __Pyx_PyInt_As_unsigned_int(__pyx_t_5); if (unlikely((__pyx_t_6 == (unsigned int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_b = __pyx_t_6;
    __pyx_v_i = __pyx_t_1;
    __pyx_t_1 = (__pyx_t_1 + 1);

    /* "_gldraw.pyx":128
 *     cdef unsigned int i, b #px+
 *     for i, b in enumerate(blocks):
 *         cube.blocks[i].transformation = cube.transformations[b]             # <<<<<<<<<<<<<<
 *         cube.blocks[i].in_motion = False
 * 
 */
    (__pyx_v_7_gldraw_cube.blocks[__pyx_v_i]).transformation = (__pyx_v_7_gldraw_cube.transformations[__pyx_v_b]);

    /* "_gldraw.pyx":129
 *     for i, b in enumerate(blocks):
 *         cube.blocks[i].transformation = cube.transformations[b]
 *         cube.blocks[i].in_motion = False             # <<<<<<<<<<<<<<
 * 
 * cpdef gl_set_atlas_texture(int x, int w, int h, bytes data): #px/
 */
    (__pyx_v_7_gldraw_cube.blocks[__pyx_v_i]).in_motion = 0;

    /* "_gldraw.pyx":127
 * def set_transformations(blocks):
 *     cdef unsigned int i, b #px+
 *     for i, b in enumerate(blocks):             # <<<<<<<<<<<<<<
 *         cube.blocks[i].transformation = cube.transformations[b]
 *         cube.blocks[i].in_motion = False
 */
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "_gldraw.pyx":125
 *     animation.angle = animation.angle_max = 0
 * 
 * def set_transformations(blocks):             # <<<<<<<<<<<<<<
 *     cdef unsigned int i, b #px+
 *     for i, b in enumerate(blocks):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("_gldraw.set_transformations", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_gldraw.pyx":131
 *         cube.blocks[i].in_motion = False
 * 
 * cpdef gl_set_atlas_texture(int x, int w, int h, bytes data): #px/             # <<<<<<<<<<<<<<
 * #def gl_set_atlas_texture(x, w, h, data):
 *     glTexSubImage2D(GL_TEXTURE_2D, 0, x, 0, w, h, GL_RGBA, GL_UNSIGNED_BYTE, <char*>data) #px/
 */

static PyObject *__pyx_pw_7_gldraw_5gl_set_atlas_texture(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_7_gldraw_gl_set_atlas_texture(int __pyx_v_x, int __pyx_v_w, int __pyx_v_h, PyObject *__pyx_v_data, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  char *__pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("gl_set_atlas_texture", 0);

  /* "_gldraw.pyx":133
 * cpdef gl_set_atlas_texture(int x, int w, int h, bytes data): #px/
 * #def gl_set_atlas_texture(x, w, h, data):
 *     glTexSubImage2D(GL_TEXTURE_2D, 0, x, 0, w, h, GL_RGBA, GL_UNSIGNED_BYTE, <char*>data) #px/             # <<<<<<<<<<<<<<
 *     #glTexSubImage2D(GL_TEXTURE_2D, 0, x, 0, w, h, GL_RGBA, GL_UNSIGNED_BYTE, data)
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_AsString(__pyx_v_data); if (unlikely((!__pyx_t_1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 133; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  glTexSubImage2D(GL_TEXTURE_2D, 0, __pyx_v_x, 0, __pyx_v_w, __pyx_v_h, GL_RGBA, GL_UNSIGNED_BYTE, ((char *)__pyx_t_1));

  /* "_gldraw.pyx":131
 *         cube.blocks[i].in_motion = False
 * 
 * cpdef gl_set_atlas_texture(int x, int w, int h, bytes data): #px/             # <<<<<<<<<<<<<<
 * #def gl_set_atlas_texture(x, w, h, data):
 *     glTexSubImage2D(GL_TEXTURE_2D, 0, x, 0, w, h, GL_RGBA, GL_UNSIGNED_BYTE, <char*>data) #px/
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("_gldraw.gl_set_atlas_texture", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_7_gldraw_5gl_set_atlas_texture(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_7_gldraw_5gl_set_atlas_texture(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_v_x;
  int __pyx_v_w;
  int __pyx_v_h;
  PyObject *__pyx_v_data = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("gl_set_atlas_texture (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,&__pyx_n_s_w,&__pyx_n_s_h,&__pyx_n_s_data,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_w)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("gl_set_atlas_texture", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 131; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_h)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("gl_set_atlas_texture", 1, 4, 4, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 131; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("gl_set_atlas_texture", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 131; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "gl_set_atlas_texture") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 131; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_x = __Pyx_PyInt_As_int(values[0]); if (unlikely((__pyx_v_x == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 131; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_w = __Pyx_PyInt_As_int(values[1]); if (unlikely((__pyx_v_w == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 131; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_h = __Pyx_PyInt_As_int(values[2]); if (unlikely((__pyx_v_h == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 131; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_data = ((PyObject*)values[3]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("gl_set_atlas_texture", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 131; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("_gldraw.gl_set_atlas_texture", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), (&PyBytes_Type), 1, "data", 1))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 131; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_7_gldraw_4gl_set_atlas_texture(__pyx_self, __pyx_v_x, __pyx_v_w, __pyx_v_h, __pyx_v_data);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7_gldraw_4gl_set_atlas_texture(CYTHON_UNUSED PyObject *__pyx_self, int __pyx_v_x, int __pyx_v_w, int __pyx_v_h, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("gl_set_atlas_texture", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_7_gldraw_gl_set_atlas_texture(__pyx_v_x, __pyx_v_w, __pyx_v_h, __pyx_v_data, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 131; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("_gldraw.gl_set_atlas_texture", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_gldraw.pyx":136
 *     #glTexSubImage2D(GL_TEXTURE_2D, 0, x, 0, w, h, GL_RGBA, GL_UNSIGNED_BYTE, data)
 * 
 * cpdef set_animation_start(blocks, float angle_max, float axisx, float axisy, float axisz): #px/             # <<<<<<<<<<<<<<
 * #def set_animation_start(blocks, angle_max, axisx, axisy, axisz):
 *     animation.angle = 0.0
 */

static PyObject *__pyx_pw_7_gldraw_7set_animation_start(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_7_gldraw_set_animation_start(PyObject *__pyx_v_blocks, float __pyx_v_angle_max, float __pyx_v_axisx, float __pyx_v_axisy, float __pyx_v_axisz, CYTHON_UNUSED int __pyx_skip_dispatch) {
  int __pyx_v_block_id;
  CYTHON_UNUSED PyObject *__pyx_v_unused_block = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  float __pyx_t_1;
  float __pyx_t_2;
  float __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *(*__pyx_t_6)(PyObject *);
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *(*__pyx_t_11)(PyObject *);
  int __pyx_t_12;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_animation_start", 0);

  /* "_gldraw.pyx":138
 * cpdef set_animation_start(blocks, float angle_max, float axisx, float axisy, float axisz): #px/
 * #def set_animation_start(blocks, angle_max, axisx, axisy, axisz):
 *     animation.angle = 0.0             # <<<<<<<<<<<<<<
 *     animation.angle_max = angle_max
 *     animation.rotation_x, animation.rotation_y, animation.rotation_z = axisx, axisy, axisz
 */
  __pyx_v_7_gldraw_animation.angle = 0.0;

  /* "_gldraw.pyx":139
 * #def set_animation_start(blocks, angle_max, axisx, axisy, axisz):
 *     animation.angle = 0.0
 *     animation.angle_max = angle_max             # <<<<<<<<<<<<<<
 *     animation.rotation_x, animation.rotation_y, animation.rotation_z = axisx, axisy, axisz
 *     matrix_set_identity(animation.rotation_matrix)
 */
  __pyx_v_7_gldraw_animation.angle_max = __pyx_v_angle_max;

  /* "_gldraw.pyx":140
 *     animation.angle = 0.0
 *     animation.angle_max = angle_max
 *     animation.rotation_x, animation.rotation_y, animation.rotation_z = axisx, axisy, axisz             # <<<<<<<<<<<<<<
 *     matrix_set_identity(animation.rotation_matrix)
 *     cdef int block_id #px+
 */
  __pyx_t_1 = __pyx_v_axisx;
  __pyx_t_2 = __pyx_v_axisy;
  __pyx_t_3 = __pyx_v_axisz;
  __pyx_v_7_gldraw_animation.rotation_x = __pyx_t_1;
  __pyx_v_7_gldraw_animation.rotation_y = __pyx_t_2;
  __pyx_v_7_gldraw_animation.rotation_z = __pyx_t_3;

  /* "_gldraw.pyx":141
 *     animation.angle_max = angle_max
 *     animation.rotation_x, animation.rotation_y, animation.rotation_z = axisx, axisy, axisz
 *     matrix_set_identity(animation.rotation_matrix)             # <<<<<<<<<<<<<<
 *     cdef int block_id #px+
 *     cdef bint selected #px+
 */
  __pyx_t_4 = __pyx_f_7_gldraw_matrix_set_identity(__pyx_v_7_gldraw_animation.rotation_matrix); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 141; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "_gldraw.pyx":144
 *     cdef int block_id #px+
 *     cdef bint selected #px+
 *     for block_id, unused_block in blocks:             # <<<<<<<<<<<<<<
 *         cube.blocks[block_id].in_motion = True
 * 
 */
  if (likely(PyList_CheckExact(__pyx_v_blocks)) || PyTuple_CheckExact(__pyx_v_blocks)) {
    __pyx_t_4 = __pyx_v_blocks; __Pyx_INCREF(__pyx_t_4); __pyx_t_5 = 0;
    __pyx_t_6 = NULL;
  } else {
    __pyx_t_5 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_v_blocks); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 144; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 144; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  for (;;) {
    if (likely(!__pyx_t_6)) {
      if (likely(PyList_CheckExact(__pyx_t_4))) {
        if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_7 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_7); __pyx_t_5++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 144; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_7 = PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 144; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #endif
      } else {
        if (__pyx_t_5 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_7 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_7); __pyx_t_5++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 144; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_7 = PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 144; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #endif
      }
    } else {
      __pyx_t_7 = __pyx_t_6(__pyx_t_4);
      if (unlikely(!__pyx_t_7)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 144; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_7);
    }
    if ((likely(PyTuple_CheckExact(__pyx_t_7))) || (PyList_CheckExact(__pyx_t_7))) {
      PyObject* sequence = __pyx_t_7;
      #if CYTHON_COMPILING_IN_CPYTHON
      Py_ssize_t size = Py_SIZE(sequence);
      #else
      Py_ssize_t size = PySequence_Size(sequence);
      #endif
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 144; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      #if CYTHON_COMPILING_IN_CPYTHON
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_8 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_9 = PyTuple_GET_ITEM(sequence, 1); 
      } else {
        __pyx_t_8 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_9 = PyList_GET_ITEM(sequence, 1); 
      }
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_9);
      #else
      __pyx_t_8 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 144; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_9 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 144; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      #endif
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_10 = PyObject_GetIter(__pyx_t_7); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 144; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_11 = Py_TYPE(__pyx_t_10)->tp_iternext;
      index = 0; __pyx_t_8 = __pyx_t_11(__pyx_t_10); if (unlikely(!__pyx_t_8)) goto __pyx_L5_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_8);
      index = 1; __pyx_t_9 = __pyx_t_11(__pyx_t_10); if (unlikely(!__pyx_t_9)) goto __pyx_L5_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_9);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_11(__pyx_t_10), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 144; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_t_11 = NULL;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      goto __pyx_L6_unpacking_done;
      __pyx_L5_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_11 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 144; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_L6_unpacking_done:;
    }
    __pyx_t_12 = __Pyx_PyInt_As_int(__pyx_t_8); if (unlikely((__pyx_t_12 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 144; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_v_block_id = __pyx_t_12;
    __Pyx_XDECREF_SET(__pyx_v_unused_block, __pyx_t_9);
    __pyx_t_9 = 0;

    /* "_gldraw.pyx":145
 *     cdef bint selected #px+
 *     for block_id, unused_block in blocks:
 *         cube.blocks[block_id].in_motion = True             # <<<<<<<<<<<<<<
 * 
 * cdef void _update_animation_rotation_matrix(): #px/
 */
    (__pyx_v_7_gldraw_cube.blocks[__pyx_v_block_id]).in_motion = 1;

    /* "_gldraw.pyx":144
 *     cdef int block_id #px+
 *     cdef bint selected #px+
 *     for block_id, unused_block in blocks:             # <<<<<<<<<<<<<<
 *         cube.blocks[block_id].in_motion = True
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "_gldraw.pyx":136
 *     #glTexSubImage2D(GL_TEXTURE_2D, 0, x, 0, w, h, GL_RGBA, GL_UNSIGNED_BYTE, data)
 * 
 * cpdef set_animation_start(blocks, float angle_max, float axisx, float axisy, float axisz): #px/             # <<<<<<<<<<<<<<
 * #def set_animation_start(blocks, angle_max, axisx, axisy, axisz):
 *     animation.angle = 0.0
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("_gldraw.set_animation_start", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_unused_block);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_7_gldraw_7set_animation_start(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_7_gldraw_7set_animation_start(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_blocks = 0;
  float __pyx_v_angle_max;
  float __pyx_v_axisx;
  float __pyx_v_axisy;
  float __pyx_v_axisz;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_animation_start (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_blocks,&__pyx_n_s_angle_max,&__pyx_n_s_axisx,&__pyx_n_s_axisy,&__pyx_n_s_axisz,0};
    PyObject* values[5] = {0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_blocks)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_angle_max)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("set_animation_start", 1, 5, 5, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 136; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_axisx)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("set_animation_start", 1, 5, 5, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 136; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_axisy)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("set_animation_start", 1, 5, 5, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 136; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_axisz)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("set_animation_start", 1, 5, 5, 4); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 136; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "set_animation_start") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 136; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 5) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
    }
    __pyx_v_blocks = values[0];
    __pyx_v_angle_max = __pyx_PyFloat_AsFloat(values[1]); if (unlikely((__pyx_v_angle_max == (float)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 136; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_axisx = __pyx_PyFloat_AsFloat(values[2]); if (unlikely((__pyx_v_axisx == (float)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 136; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_axisy = __pyx_PyFloat_AsFloat(values[3]); if (unlikely((__pyx_v_axisy == (float)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 136; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_axisz = __pyx_PyFloat_AsFloat(values[4]); if (unlikely((__pyx_v_axisz == (float)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 136; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("set_animation_start", 1, 5, 5, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 136; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("_gldraw.set_animation_start", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7_gldraw_6set_animation_start(__pyx_self, __pyx_v_blocks, __pyx_v_angle_max, __pyx_v_axisx, __pyx_v_axisy, __pyx_v_axisz);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7_gldraw_6set_animation_start(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_blocks, float __pyx_v_angle_max, float __pyx_v_axisx, float __pyx_v_axisy, float __pyx_v_axisz) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_animation_start", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_7_gldraw_set_animation_start(__pyx_v_blocks, __pyx_v_angle_max, __pyx_v_axisx, __pyx_v_axisy, __pyx_v_axisz, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 136; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("_gldraw.set_animation_start", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_gldraw.pyx":147
 *         cube.blocks[block_id].in_motion = True
 * 
 * cdef void _update_animation_rotation_matrix(): #px/             # <<<<<<<<<<<<<<
 * #def _update_animation_rotation_matrix():
 *     cdef float angle = animation.angle / 180. * M_PI #px/
 */

static void __pyx_f_7_gldraw__update_animation_rotation_matrix(void) {
  float __pyx_v_angle;
  float __pyx_v_x;
  float __pyx_v_y;
  float __pyx_v_z;
  float __pyx_v_c;
  float __pyx_v_s;
  __Pyx_RefNannyDeclarations
  float __pyx_t_1;
  __Pyx_RefNannySetupContext("_update_animation_rotation_matrix", 0);

  /* "_gldraw.pyx":149
 * cdef void _update_animation_rotation_matrix(): #px/
 * #def _update_animation_rotation_matrix():
 *     cdef float angle = animation.angle / 180. * M_PI #px/             # <<<<<<<<<<<<<<
 *     #angle = radians(animation.angle)
 *     cdef float x, y, z #px+
 */
  __pyx_v_angle = ((((double)__pyx_v_7_gldraw_animation.angle) / 180.) * M_PI);

  /* "_gldraw.pyx":152
 *     #angle = radians(animation.angle)
 *     cdef float x, y, z #px+
 *     x = animation.rotation_x             # <<<<<<<<<<<<<<
 *     y = animation.rotation_y
 *     z = animation.rotation_z
 */
  __pyx_t_1 = __pyx_v_7_gldraw_animation.rotation_x;
  __pyx_v_x = __pyx_t_1;

  /* "_gldraw.pyx":153
 *     cdef float x, y, z #px+
 *     x = animation.rotation_x
 *     y = animation.rotation_y             # <<<<<<<<<<<<<<
 *     z = animation.rotation_z
 *     cdef float c, s #px+
 */
  __pyx_t_1 = __pyx_v_7_gldraw_animation.rotation_y;
  __pyx_v_y = __pyx_t_1;

  /* "_gldraw.pyx":154
 *     x = animation.rotation_x
 *     y = animation.rotation_y
 *     z = animation.rotation_z             # <<<<<<<<<<<<<<
 *     cdef float c, s #px+
 *     c = cos(angle)
 */
  __pyx_t_1 = __pyx_v_7_gldraw_animation.rotation_z;
  __pyx_v_z = __pyx_t_1;

  /* "_gldraw.pyx":156
 *     z = animation.rotation_z
 *     cdef float c, s #px+
 *     c = cos(angle)             # <<<<<<<<<<<<<<
 *     s = sin(angle)
 * 
 */
  __pyx_v_c = cos(__pyx_v_angle);

  /* "_gldraw.pyx":157
 *     cdef float c, s #px+
 *     c = cos(angle)
 *     s = sin(angle)             # <<<<<<<<<<<<<<
 * 
 *     animation.rotation_matrix[0][0] = x*x*(1-c) + c
 */
  __pyx_v_s = sin(__pyx_v_angle);

  /* "_gldraw.pyx":159
 *     s = sin(angle)
 * 
 *     animation.rotation_matrix[0][0] = x*x*(1-c) + c             # <<<<<<<<<<<<<<
 *     animation.rotation_matrix[0][1] = x*y*(1-c) + z*s
 *     animation.rotation_matrix[0][2] = x*z*(1-c) - y*s
 */
  ((__pyx_v_7_gldraw_animation.rotation_matrix[0])[0]) = (((__pyx_v_x * __pyx_v_x) * (1.0 - __pyx_v_c)) + __pyx_v_c);

  /* "_gldraw.pyx":160
 * 
 *     animation.rotation_matrix[0][0] = x*x*(1-c) + c
 *     animation.rotation_matrix[0][1] = x*y*(1-c) + z*s             # <<<<<<<<<<<<<<
 *     animation.rotation_matrix[0][2] = x*z*(1-c) - y*s
 *     animation.rotation_matrix[1][0] = y*x*(1-c) - z*s
 */
  ((__pyx_v_7_gldraw_animation.rotation_matrix[0])[1]) = (((__pyx_v_x * __pyx_v_y) * (1.0 - __pyx_v_c)) + (__pyx_v_z * __pyx_v_s));

  /* "_gldraw.pyx":161
 *     animation.rotation_matrix[0][0] = x*x*(1-c) + c
 *     animation.rotation_matrix[0][1] = x*y*(1-c) + z*s
 *     animation.rotation_matrix[0][2] = x*z*(1-c) - y*s             # <<<<<<<<<<<<<<
 *     animation.rotation_matrix[1][0] = y*x*(1-c) - z*s
 *     animation.rotation_matrix[1][1] = y*y*(1-c) + c
 */
  ((__pyx_v_7_gldraw_animation.rotation_matrix[0])[2]) = (((__pyx_v_x * __pyx_v_z) * (1.0 - __pyx_v_c)) - (__pyx_v_y * __pyx_v_s));

  /* "_gldraw.pyx":162
 *     animation.rotation_matrix[0][1] = x*y*(1-c) + z*s
 *     animation.rotation_matrix[0][2] = x*z*(1-c) - y*s
 *     animation.rotation_matrix[1][0] = y*x*(1-c) - z*s             # <<<<<<<<<<<<<<
 *     animation.rotation_matrix[1][1] = y*y*(1-c) + c
 *     animation.rotation_matrix[1][2] = y*z*(1-c) + x*s
 */
  ((__pyx_v_7_gldraw_animation.rotation_matrix[1])[0]) = (((__pyx_v_y * __pyx_v_x) * (1.0 - __pyx_v_c)) - (__pyx_v_z * __pyx_v_s));

  /* "_gldraw.pyx":163
 *     animation.rotation_matrix[0][2] = x*z*(1-c) - y*s
 *     animation.rotation_matrix[1][0] = y*x*(1-c) - z*s
 *     animation.rotation_matrix[1][1] = y*y*(1-c) + c             # <<<<<<<<<<<<<<
 *     animation.rotation_matrix[1][2] = y*z*(1-c) + x*s
 *     animation.rotation_matrix[2][0] = x*z*(1-c) + y*s
 */
  ((__pyx_v_7_gldraw_animation.rotation_matrix[1])[1]) = (((__pyx_v_y * __pyx_v_y) * (1.0 - __pyx_v_c)) + __pyx_v_c);

  /* "_gldraw.pyx":164
 *     animation.rotation_matrix[1][0] = y*x*(1-c) - z*s
 *     animation.rotation_matrix[1][1] = y*y*(1-c) + c
 *     animation.rotation_matrix[1][2] = y*z*(1-c) + x*s             # <<<<<<<<<<<<<<
 *     animation.rotation_matrix[2][0] = x*z*(1-c) + y*s
 *     animation.rotation_matrix[2][1] = y*z*(1-c) - x*s
 */
  ((__pyx_v_7_gldraw_animation.rotation_matrix[1])[2]) = (((__pyx_v_y * __pyx_v_z) * (1.0 - __pyx_v_c)) + (__pyx_v_x * __pyx_v_s));

  /* "_gldraw.pyx":165
 *     animation.rotation_matrix[1][1] = y*y*(1-c) + c
 *     animation.rotation_matrix[1][2] = y*z*(1-c) + x*s
 *     animation.rotation_matrix[2][0] = x*z*(1-c) + y*s             # <<<<<<<<<<<<<<
 *     animation.rotation_matrix[2][1] = y*z*(1-c) - x*s
 *     animation.rotation_matrix[2][2] = z*z*(1-c) + c
 */
  ((__pyx_v_7_gldraw_animation.rotation_matrix[2])[0]) = (((__pyx_v_x * __pyx_v_z) * (1.0 - __pyx_v_c)) + (__pyx_v_y * __pyx_v_s));

  /* "_gldraw.pyx":166
 *     animation.rotation_matrix[1][2] = y*z*(1-c) + x*s
 *     animation.rotation_matrix[2][0] = x*z*(1-c) + y*s
 *     animation.rotation_matrix[2][1] = y*z*(1-c) - x*s             # <<<<<<<<<<<<<<
 *     animation.rotation_matrix[2][2] = z*z*(1-c) + c
 * 
 */
  ((__pyx_v_7_gldraw_animation.rotation_matrix[2])[1]) = (((__pyx_v_y * __pyx_v_z) * (1.0 - __pyx_v_c)) - (__pyx_v_x * __pyx_v_s));

  /* "_gldraw.pyx":167
 *     animation.rotation_matrix[2][0] = x*z*(1-c) + y*s
 *     animation.rotation_matrix[2][1] = y*z*(1-c) - x*s
 *     animation.rotation_matrix[2][2] = z*z*(1-c) + c             # <<<<<<<<<<<<<<
 * 
 * cpdef set_animation_next(float increment): #px/
 */
  ((__pyx_v_7_gldraw_animation.rotation_matrix[2])[2]) = (((__pyx_v_z * __pyx_v_z) * (1.0 - __pyx_v_c)) + __pyx_v_c);

  /* "_gldraw.pyx":147
 *         cube.blocks[block_id].in_motion = True
 * 
 * cdef void _update_animation_rotation_matrix(): #px/             # <<<<<<<<<<<<<<
 * #def _update_animation_rotation_matrix():
 *     cdef float angle = animation.angle / 180. * M_PI #px/
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "_gldraw.pyx":169
 *     animation.rotation_matrix[2][2] = z*z*(1-c) + c
 * 
 * cpdef set_animation_next(float increment): #px/             # <<<<<<<<<<<<<<
 * #def set_animation_next(increment):
 *     animation.angle -= increment
 */

static PyObject *__pyx_pw_7_gldraw_9set_animation_next(PyObject *__pyx_self, PyObject *__pyx_arg_increment); /*proto*/
static PyObject *__pyx_f_7_gldraw_set_animation_next(float __pyx_v_increment, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_animation_next", 0);

  /* "_gldraw.pyx":171
 * cpdef set_animation_next(float increment): #px/
 * #def set_animation_next(increment):
 *     animation.angle -= increment             # <<<<<<<<<<<<<<
 *     _update_animation_rotation_matrix()
 *     return abs(animation.angle) < animation.angle_max
 */
  __pyx_v_7_gldraw_animation.angle = (__pyx_v_7_gldraw_animation.angle - __pyx_v_increment);

  /* "_gldraw.pyx":172
 * #def set_animation_next(increment):
 *     animation.angle -= increment
 *     _update_animation_rotation_matrix()             # <<<<<<<<<<<<<<
 *     return abs(animation.angle) < animation.angle_max
 * 
 */
  __pyx_f_7_gldraw__update_animation_rotation_matrix();

  /* "_gldraw.pyx":173
 *     animation.angle -= increment
 *     _update_animation_rotation_matrix()
 *     return abs(animation.angle) < animation.angle_max             # <<<<<<<<<<<<<<
 * 
 * cdef void _mult_matrix(mat4 &dest, mat4 &src1, mat4 &src2): #px/
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong((fabsf(__pyx_v_7_gldraw_animation.angle) < __pyx_v_7_gldraw_animation.angle_max)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 173; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "_gldraw.pyx":169
 *     animation.rotation_matrix[2][2] = z*z*(1-c) + c
 * 
 * cpdef set_animation_next(float increment): #px/             # <<<<<<<<<<<<<<
 * #def set_animation_next(increment):
 *     animation.angle -= increment
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("_gldraw.set_animation_next", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_7_gldraw_9set_animation_next(PyObject *__pyx_self, PyObject *__pyx_arg_increment); /*proto*/
static PyObject *__pyx_pw_7_gldraw_9set_animation_next(PyObject *__pyx_self, PyObject *__pyx_arg_increment) {
  float __pyx_v_increment;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_animation_next (wrapper)", 0);
  assert(__pyx_arg_increment); {
    __pyx_v_increment = __pyx_PyFloat_AsFloat(__pyx_arg_increment); if (unlikely((__pyx_v_increment == (float)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 169; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("_gldraw.set_animation_next", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7_gldraw_8set_animation_next(__pyx_self, ((float)__pyx_v_increment));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7_gldraw_8set_animation_next(CYTHON_UNUSED PyObject *__pyx_self, float __pyx_v_increment) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_animation_next", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_7_gldraw_set_animation_next(__pyx_v_increment, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 169; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("_gldraw.set_animation_next", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_gldraw.pyx":175
 *     return abs(animation.angle) < animation.angle_max
 * 
 * cdef void _mult_matrix(mat4 &dest, mat4 &src1, mat4 &src2): #px/             # <<<<<<<<<<<<<<
 * #def _mult_matrix(dest, src1, src2):
 *     cdef int i, j, k #px+
 */

static void __pyx_f_7_gldraw__mult_matrix(__pyx_t_7_gldraw_vec4 *__pyx_v_dest, __pyx_t_7_gldraw_vec4 *__pyx_v_src1, __pyx_t_7_gldraw_vec4 *__pyx_v_src2) {
  int __pyx_v_i;
  int __pyx_v_j;
  int __pyx_v_k;
  float __pyx_v_sum_;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("_mult_matrix", 0);

  /* "_gldraw.pyx":179
 *     cdef int i, j, k #px+
 *     cdef float sum_ #px+
 *     for j in range(4):             # <<<<<<<<<<<<<<
 *         for i in range(4):
 *             sum_ = 0.
 */
  for (__pyx_t_1 = 0; __pyx_t_1 < 4; __pyx_t_1+=1) {
    __pyx_v_j = __pyx_t_1;

    /* "_gldraw.pyx":180
 *     cdef float sum_ #px+
 *     for j in range(4):
 *         for i in range(4):             # <<<<<<<<<<<<<<
 *             sum_ = 0.
 *             for k in range(4):
 */
    for (__pyx_t_2 = 0; __pyx_t_2 < 4; __pyx_t_2+=1) {
      __pyx_v_i = __pyx_t_2;

      /* "_gldraw.pyx":181
 *     for j in range(4):
 *         for i in range(4):
 *             sum_ = 0.             # <<<<<<<<<<<<<<
 *             for k in range(4):
 *                 sum_ += src1[k][i] * src2[j][k]
 */
      __pyx_v_sum_ = 0.;

      /* "_gldraw.pyx":182
 *         for i in range(4):
 *             sum_ = 0.
 *             for k in range(4):             # <<<<<<<<<<<<<<
 *                 sum_ += src1[k][i] * src2[j][k]
 *             dest[j][i] = sum_
 */
      for (__pyx_t_3 = 0; __pyx_t_3 < 4; __pyx_t_3+=1) {
        __pyx_v_k = __pyx_t_3;

        /* "_gldraw.pyx":183
 *             sum_ = 0.
 *             for k in range(4):
 *                 sum_ += src1[k][i] * src2[j][k]             # <<<<<<<<<<<<<<
 *             dest[j][i] = sum_
 * 
 */
        __pyx_v_sum_ = (__pyx_v_sum_ + (((__pyx_v_src1[__pyx_v_k])[__pyx_v_i]) * ((__pyx_v_src2[__pyx_v_j])[__pyx_v_k])));
      }

      /* "_gldraw.pyx":184
 *             for k in range(4):
 *                 sum_ += src1[k][i] * src2[j][k]
 *             dest[j][i] = sum_             # <<<<<<<<<<<<<<
 * 
 * cdef void gl_draw_cube(): #pxd/
 */
      ((__pyx_v_dest[__pyx_v_j])[__pyx_v_i]) = __pyx_v_sum_;
    }
  }

  /* "_gldraw.pyx":175
 *     return abs(animation.angle) < animation.angle_max
 * 
 * cdef void _mult_matrix(mat4 &dest, mat4 &src1, mat4 &src2): #px/             # <<<<<<<<<<<<<<
 * #def _mult_matrix(dest, src1, src2):
 *     cdef int i, j, k #px+
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "_gldraw.pyx":186
 *             dest[j][i] = sum_
 * 
 * cdef void gl_draw_cube(): #pxd/             # <<<<<<<<<<<<<<
 * #def gl_draw_cube():
 *     cdef unsigned int i #px+
 */

static void __pyx_f_7_gldraw_gl_draw_cube(void) {
  unsigned int __pyx_v_i;
  __pyx_t_7_gldraw_mat4 __pyx_v_object_matrix;
  __Pyx_RefNannyDeclarations
  unsigned int __pyx_t_1;
  unsigned int __pyx_t_2;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("gl_draw_cube", 0);

  /* "_gldraw.pyx":191
 *     cdef mat4 object_matrix #px/
 *     #object_matrix = mat4()
 *     for i in range(cube.number_blocks):             # <<<<<<<<<<<<<<
 *         if cube.blocks[i].in_motion:
 *             _mult_matrix(object_matrix, animation.rotation_matrix, cube.blocks[i].transformation)
 */
  __pyx_t_1 = __pyx_v_7_gldraw_cube.number_blocks;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "_gldraw.pyx":192
 *     #object_matrix = mat4()
 *     for i in range(cube.number_blocks):
 *         if cube.blocks[i].in_motion:             # <<<<<<<<<<<<<<
 *             _mult_matrix(object_matrix, animation.rotation_matrix, cube.blocks[i].transformation)
 *             glUniformMatrix4fv(cube.object_location, 1, GL_FALSE, &object_matrix[0][0]) #px/
 */
    __pyx_t_3 = ((__pyx_v_7_gldraw_cube.blocks[__pyx_v_i]).in_motion != 0);
    if (__pyx_t_3) {

      /* "_gldraw.pyx":193
 *     for i in range(cube.number_blocks):
 *         if cube.blocks[i].in_motion:
 *             _mult_matrix(object_matrix, animation.rotation_matrix, cube.blocks[i].transformation)             # <<<<<<<<<<<<<<
 *             glUniformMatrix4fv(cube.object_location, 1, GL_FALSE, &object_matrix[0][0]) #px/
 *             #glUniformMatrix4fv(cube.object_location, 1, GL_FALSE, object_matrix)
 */
      __pyx_f_7_gldraw__mult_matrix(__pyx_v_object_matrix, __pyx_v_7_gldraw_animation.rotation_matrix, (__pyx_v_7_gldraw_cube.blocks[__pyx_v_i]).transformation);

      /* "_gldraw.pyx":194
 *         if cube.blocks[i].in_motion:
 *             _mult_matrix(object_matrix, animation.rotation_matrix, cube.blocks[i].transformation)
 *             glUniformMatrix4fv(cube.object_location, 1, GL_FALSE, &object_matrix[0][0]) #px/             # <<<<<<<<<<<<<<
 *             #glUniformMatrix4fv(cube.object_location, 1, GL_FALSE, object_matrix)
 *         else:
 */
      glUniformMatrix4fv(__pyx_v_7_gldraw_cube.object_location, 1, GL_FALSE, (&((__pyx_v_object_matrix[0])[0])));
      goto __pyx_L5;
    }
    /*else*/ {

      /* "_gldraw.pyx":197
 *             #glUniformMatrix4fv(cube.object_location, 1, GL_FALSE, object_matrix)
 *         else:
 *             glUniformMatrix4fv(cube.object_location, 1, GL_FALSE, &cube.blocks[i].transformation[0][0]) #px/             # <<<<<<<<<<<<<<
 *             #glUniformMatrix4fv(cube.object_location, 1, GL_FALSE, cube.blocks[i].transformation)
 *         glDrawArrays(GL_TRIANGLES, cube.blocks[i].idx_triangles, cube.blocks[i].cnt_triangles)
 */
      glUniformMatrix4fv(__pyx_v_7_gldraw_cube.object_location, 1, GL_FALSE, (&(((__pyx_v_7_gldraw_cube.blocks[__pyx_v_i]).transformation[0])[0])));
    }
    __pyx_L5:;

    /* "_gldraw.pyx":199
 *             glUniformMatrix4fv(cube.object_location, 1, GL_FALSE, &cube.blocks[i].transformation[0][0]) #px/
 *             #glUniformMatrix4fv(cube.object_location, 1, GL_FALSE, cube.blocks[i].transformation)
 *         glDrawArrays(GL_TRIANGLES, cube.blocks[i].idx_triangles, cube.blocks[i].cnt_triangles)             # <<<<<<<<<<<<<<
 * 
 * cdef void gl_pick_cube(): #pxd/
 */
    glDrawArrays(GL_TRIANGLES, (__pyx_v_7_gldraw_cube.blocks[__pyx_v_i]).idx_triangles, (__pyx_v_7_gldraw_cube.blocks[__pyx_v_i]).cnt_triangles);
  }

  /* "_gldraw.pyx":186
 *             dest[j][i] = sum_
 * 
 * cdef void gl_draw_cube(): #pxd/             # <<<<<<<<<<<<<<
 * #def gl_draw_cube():
 *     cdef unsigned int i #px+
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "_gldraw.pyx":201
 *         glDrawArrays(GL_TRIANGLES, cube.blocks[i].idx_triangles, cube.blocks[i].cnt_triangles)
 * 
 * cdef void gl_pick_cube(): #pxd/             # <<<<<<<<<<<<<<
 * #def gl_pick_cube():
 *     glDrawArrays(GL_TRIANGLES, 0, cube.cnt_pick)
 */

static void __pyx_f_7_gldraw_gl_pick_cube(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("gl_pick_cube", 0);

  /* "_gldraw.pyx":203
 * cdef void gl_pick_cube(): #pxd/
 * #def gl_pick_cube():
 *     glDrawArrays(GL_TRIANGLES, 0, cube.cnt_pick)             # <<<<<<<<<<<<<<
 * 
 * cdef void gl_init_buffers(): #pxd/
 */
  glDrawArrays(GL_TRIANGLES, 0, __pyx_v_7_gldraw_cube.cnt_pick);

  /* "_gldraw.pyx":201
 *         glDrawArrays(GL_TRIANGLES, cube.blocks[i].idx_triangles, cube.blocks[i].cnt_triangles)
 * 
 * cdef void gl_pick_cube(): #pxd/             # <<<<<<<<<<<<<<
 * #def gl_pick_cube():
 *     glDrawArrays(GL_TRIANGLES, 0, cube.cnt_pick)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "_gldraw.pyx":205
 *     glDrawArrays(GL_TRIANGLES, 0, cube.cnt_pick)
 * 
 * cdef void gl_init_buffers(): #pxd/             # <<<<<<<<<<<<<<
 * #def gl_init_buffers():
 *     glGenBuffers(1, &cube.glbuffer) #px/
 */

static void __pyx_f_7_gldraw_gl_init_buffers(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("gl_init_buffers", 0);

  /* "_gldraw.pyx":207
 * cdef void gl_init_buffers(): #pxd/
 * #def gl_init_buffers():
 *     glGenBuffers(1, &cube.glbuffer) #px/             # <<<<<<<<<<<<<<
 *     #cube.glbuffer = glGenBuffers(1)
 * 
 */
  glGenBuffers(1, (&__pyx_v_7_gldraw_cube.glbuffer));

  /* "_gldraw.pyx":205
 *     glDrawArrays(GL_TRIANGLES, 0, cube.cnt_pick)
 * 
 * cdef void gl_init_buffers(): #pxd/             # <<<<<<<<<<<<<<
 * #def gl_init_buffers():
 *     glGenBuffers(1, &cube.glbuffer) #px/
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "_gldraw.pyx":210
 *     #cube.glbuffer = glGenBuffers(1)
 * 
 * cdef void gl_delete_buffers(): #pxd/             # <<<<<<<<<<<<<<
 * #def gl_delete_buffers():
 *     glBindBuffer(GL_ARRAY_BUFFER, 0)
 */

static void __pyx_f_7_gldraw_gl_delete_buffers(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("gl_delete_buffers", 0);

  /* "_gldraw.pyx":212
 * cdef void gl_delete_buffers(): #pxd/
 * #def gl_delete_buffers():
 *     glBindBuffer(GL_ARRAY_BUFFER, 0)             # <<<<<<<<<<<<<<
 *     glDeleteBuffers(1, &cube.glbuffer) #px/
 *     #glDeleteBuffers(1, [cube.glbuffer])
 */
  glBindBuffer(GL_ARRAY_BUFFER, 0);

  /* "_gldraw.pyx":213
 * #def gl_delete_buffers():
 *     glBindBuffer(GL_ARRAY_BUFFER, 0)
 *     glDeleteBuffers(1, &cube.glbuffer) #px/             # <<<<<<<<<<<<<<
 *     #glDeleteBuffers(1, [cube.glbuffer])
 *     cube.glbuffer = 0
 */
  glDeleteBuffers(1, (&__pyx_v_7_gldraw_cube.glbuffer));

  /* "_gldraw.pyx":215
 *     glDeleteBuffers(1, &cube.glbuffer) #px/
 *     #glDeleteBuffers(1, [cube.glbuffer])
 *     cube.glbuffer = 0             # <<<<<<<<<<<<<<
 * 
 * cdef void _gl_set_pointer(GLuint index, GLint size, GLenum type, GLboolean normalized, long pointer): #px/
 */
  __pyx_v_7_gldraw_cube.glbuffer = 0;

  /* "_gldraw.pyx":210
 *     #cube.glbuffer = glGenBuffers(1)
 * 
 * cdef void gl_delete_buffers(): #pxd/             # <<<<<<<<<<<<<<
 * #def gl_delete_buffers():
 *     glBindBuffer(GL_ARRAY_BUFFER, 0)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "_gldraw.pyx":217
 *     cube.glbuffer = 0
 * 
 * cdef void _gl_set_pointer(GLuint index, GLint size, GLenum type, GLboolean normalized, long pointer): #px/             # <<<<<<<<<<<<<<
 * #def _gl_set_pointer(index, size, type, normalized, pointer):
 *     glVertexAttribPointer(index, size, type, normalized, 0, <void*>pointer) #px/
 */

static void __pyx_f_7_gldraw__gl_set_pointer(__pyx_t_2gl_GLuint __pyx_v_index, __pyx_t_2gl_GLint __pyx_v_size, __pyx_t_2gl_GLenum __pyx_v_type, __pyx_t_2gl_GLboolean __pyx_v_normalized, long __pyx_v_pointer) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_gl_set_pointer", 0);

  /* "_gldraw.pyx":219
 * cdef void _gl_set_pointer(GLuint index, GLint size, GLenum type, GLboolean normalized, long pointer): #px/
 * #def _gl_set_pointer(index, size, type, normalized, pointer):
 *     glVertexAttribPointer(index, size, type, normalized, 0, <void*>pointer) #px/             # <<<<<<<<<<<<<<
 *     #glVertexAttribPointer(index, size, type, normalized, 0, ctypes.cast(pointer, ctypes.c_void_p))
 *     glEnableVertexAttribArray(index)
 */
  glVertexAttribPointer(__pyx_v_index, __pyx_v_size, __pyx_v_type, __pyx_v_normalized, 0, ((void *)__pyx_v_pointer));

  /* "_gldraw.pyx":221
 *     glVertexAttribPointer(index, size, type, normalized, 0, <void*>pointer) #px/
 *     #glVertexAttribPointer(index, size, type, normalized, 0, ctypes.cast(pointer, ctypes.c_void_p))
 *     glEnableVertexAttribArray(index)             # <<<<<<<<<<<<<<
 * 
 * cpdef gl_set_data(int nblocks, bytes vertexdata, vertexpointers, vertexinfo, transformations): #px/
 */
  glEnableVertexAttribArray(__pyx_v_index);

  /* "_gldraw.pyx":217
 *     cube.glbuffer = 0
 * 
 * cdef void _gl_set_pointer(GLuint index, GLint size, GLenum type, GLboolean normalized, long pointer): #px/             # <<<<<<<<<<<<<<
 * #def _gl_set_pointer(index, size, type, normalized, pointer):
 *     glVertexAttribPointer(index, size, type, normalized, 0, <void*>pointer) #px/
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "_gldraw.pyx":223
 *     glEnableVertexAttribArray(index)
 * 
 * cpdef gl_set_data(int nblocks, bytes vertexdata, vertexpointers, vertexinfo, transformations): #px/             # <<<<<<<<<<<<<<
 * #def gl_set_data(nblocks, vertexdata, vertexpointers, vertexinfo, transformations):
 *     assert nblocks <= MAX_BLOCKS, '{} blocks, hardcoded limit is {}'.format(
 */

static PyObject *__pyx_pw_7_gldraw_11gl_set_data(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_7_gldraw_gl_set_data(int __pyx_v_nblocks, PyObject *__pyx_v_vertexdata, PyObject *__pyx_v_vertexpointers, PyObject *__pyx_v_vertexinfo, PyObject *__pyx_v_transformations, CYTHON_UNUSED int __pyx_skip_dispatch) {
  long __pyx_v_normalpointer;
  long __pyx_v_colorpointer;
  long __pyx_v_texpospointer;
  long __pyx_v_barycpointer;
  unsigned int __pyx_v_idx_debug;
  unsigned int __pyx_v_cnt_pick;
  PyObject *__pyx_v_pickvertpointer = NULL;
  PyObject *__pyx_v_pickcolorpointer = NULL;
  PyObject *__pyx_v_cnts_block = NULL;
  unsigned int __pyx_v_idx_block;
  unsigned int __pyx_v_idx;
  unsigned int __pyx_v_i;
  unsigned int __pyx_v_j;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  Py_ssize_t __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *(*__pyx_t_9)(PyObject *);
  long __pyx_t_10;
  long __pyx_t_11;
  long __pyx_t_12;
  long __pyx_t_13;
  unsigned int __pyx_t_14;
  unsigned int __pyx_t_15;
  int __pyx_t_16;
  unsigned int __pyx_t_17;
  char *__pyx_t_18;
  Py_ssize_t __pyx_t_19;
  double __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("gl_set_data", 0);

  /* "_gldraw.pyx":225
 * cpdef gl_set_data(int nblocks, bytes vertexdata, vertexpointers, vertexinfo, transformations): #px/
 * #def gl_set_data(nblocks, vertexdata, vertexpointers, vertexinfo, transformations):
 *     assert nblocks <= MAX_BLOCKS, '{} blocks, hardcoded limit is {}'.format(             # <<<<<<<<<<<<<<
 *                                   nblocks, MAX_BLOCKS)
 *     cube.number_blocks = nblocks
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__pyx_v_nblocks <= __pyx_e_7_gldraw_MAX_BLOCKS) != 0))) {
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_blocks_hardcoded_limit_is, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 225; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);

      /* "_gldraw.pyx":226
 * #def gl_set_data(nblocks, vertexdata, vertexpointers, vertexinfo, transformations):
 *     assert nblocks <= MAX_BLOCKS, '{} blocks, hardcoded limit is {}'.format(
 *                                   nblocks, MAX_BLOCKS)             # <<<<<<<<<<<<<<
 *     cube.number_blocks = nblocks
 * 
 */
      __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_nblocks); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 226; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_e_7_gldraw_MAX_BLOCKS); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 226; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = NULL;
      __pyx_t_6 = 0;
      if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
          __pyx_t_6 = 1;
        }
      }
      __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 225; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_7);
      if (__pyx_t_5) {
        PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __Pyx_GIVEREF(__pyx_t_5); __pyx_t_5 = NULL;
      }
      PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_4);
      __pyx_t_3 = 0;
      __pyx_t_4 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 225; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "_gldraw.pyx":225
 * cpdef gl_set_data(int nblocks, bytes vertexdata, vertexpointers, vertexinfo, transformations): #px/
 * #def gl_set_data(nblocks, vertexdata, vertexpointers, vertexinfo, transformations):
 *     assert nblocks <= MAX_BLOCKS, '{} blocks, hardcoded limit is {}'.format(             # <<<<<<<<<<<<<<
 *                                   nblocks, MAX_BLOCKS)
 *     cube.number_blocks = nblocks
 */
      __pyx_t_2 = PyTuple_Pack(1, __pyx_t_1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 225; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      PyErr_SetObject(PyExc_AssertionError, __pyx_t_2);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 225; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  #endif

  /* "_gldraw.pyx":227
 *     assert nblocks <= MAX_BLOCKS, '{} blocks, hardcoded limit is {}'.format(
 *                                   nblocks, MAX_BLOCKS)
 *     cube.number_blocks = nblocks             # <<<<<<<<<<<<<<
 * 
 *     #### Create the raw GL data ####
 */
  __pyx_v_7_gldraw_cube.number_blocks = __pyx_v_nblocks;

  /* "_gldraw.pyx":232
 *     cdef long normalpointer, colorpointer, texpospointer, barycpointer #px+
 *     cdef unsigned int idx_debug, cnt_pick #px+
 *     normalpointer, colorpointer, texpospointer, barycpointer, pickvertpointer, pickcolorpointer = vertexpointers             # <<<<<<<<<<<<<<
 *     cnts_block, idx_debug, cnt_pick = vertexinfo
 * 
 */
  if ((likely(PyTuple_CheckExact(__pyx_v_vertexpointers))) || (PyList_CheckExact(__pyx_v_vertexpointers))) {
    PyObject* sequence = __pyx_v_vertexpointers;
    #if CYTHON_COMPILING_IN_CPYTHON
    Py_ssize_t size = Py_SIZE(sequence);
    #else
    Py_ssize_t size = PySequence_Size(sequence);
    #endif
    if (unlikely(size != 6)) {
      if (size > 6) __Pyx_RaiseTooManyValuesError(6);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 232; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    #if CYTHON_COMPILING_IN_CPYTHON
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_1 = PyTuple_GET_ITEM(sequence, 1); 
      __pyx_t_7 = PyTuple_GET_ITEM(sequence, 2); 
      __pyx_t_4 = PyTuple_GET_ITEM(sequence, 3); 
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 4); 
      __pyx_t_5 = PyTuple_GET_ITEM(sequence, 5); 
    } else {
      __pyx_t_2 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_1 = PyList_GET_ITEM(sequence, 1); 
      __pyx_t_7 = PyList_GET_ITEM(sequence, 2); 
      __pyx_t_4 = PyList_GET_ITEM(sequence, 3); 
      __pyx_t_3 = PyList_GET_ITEM(sequence, 4); 
      __pyx_t_5 = PyList_GET_ITEM(sequence, 5); 
    }
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_7);
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_5);
    #else
    {
      Py_ssize_t i;
      PyObject** temps[6] = {&__pyx_t_2,&__pyx_t_1,&__pyx_t_7,&__pyx_t_4,&__pyx_t_3,&__pyx_t_5};
      for (i=0; i < 6; i++) {
        PyObject* item = PySequence_ITEM(sequence, i); if (unlikely(!item)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 232; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(item);
        *(temps[i]) = item;
      }
    }
    #endif
  } else {
    Py_ssize_t index = -1;
    PyObject** temps[6] = {&__pyx_t_2,&__pyx_t_1,&__pyx_t_7,&__pyx_t_4,&__pyx_t_3,&__pyx_t_5};
    __pyx_t_8 = PyObject_GetIter(__pyx_v_vertexpointers); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 232; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = Py_TYPE(__pyx_t_8)->tp_iternext;
    for (index=0; index < 6; index++) {
      PyObject* item = __pyx_t_9(__pyx_t_8); if (unlikely(!item)) goto __pyx_L3_unpacking_failed;
      __Pyx_GOTREF(item);
      *(temps[index]) = item;
    }
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_9(__pyx_t_8), 6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 232; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_9 = NULL;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    goto __pyx_L4_unpacking_done;
    __pyx_L3_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_9 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 232; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_L4_unpacking_done:;
  }
  __pyx_t_10 = __Pyx_PyInt_As_long(__pyx_t_2); if (unlikely((__pyx_t_10 == (long)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 232; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_11 = __Pyx_PyInt_As_long(__pyx_t_1); if (unlikely((__pyx_t_11 == (long)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 232; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_12 = __Pyx_PyInt_As_long(__pyx_t_7); if (unlikely((__pyx_t_12 == (long)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 232; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_13 = __Pyx_PyInt_As_long(__pyx_t_4); if (unlikely((__pyx_t_13 == (long)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 232; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_normalpointer = __pyx_t_10;
  __pyx_v_colorpointer = __pyx_t_11;
  __pyx_v_texpospointer = __pyx_t_12;
  __pyx_v_barycpointer = __pyx_t_13;
  __pyx_v_pickvertpointer = __pyx_t_3;
  __pyx_t_3 = 0;
  __pyx_v_pickcolorpointer = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "_gldraw.pyx":233
 *     cdef unsigned int idx_debug, cnt_pick #px+
 *     normalpointer, colorpointer, texpospointer, barycpointer, pickvertpointer, pickcolorpointer = vertexpointers
 *     cnts_block, idx_debug, cnt_pick = vertexinfo             # <<<<<<<<<<<<<<
 * 
 *     cdef unsigned int idx_block, idx, i, j #px+
 */
  if ((likely(PyTuple_CheckExact(__pyx_v_vertexinfo))) || (PyList_CheckExact(__pyx_v_vertexinfo))) {
    PyObject* sequence = __pyx_v_vertexinfo;
    #if CYTHON_COMPILING_IN_CPYTHON
    Py_ssize_t size = Py_SIZE(sequence);
    #else
    Py_ssize_t size = PySequence_Size(sequence);
    #endif
    if (unlikely(size != 3)) {
      if (size > 3) __Pyx_RaiseTooManyValuesError(3);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 233; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    #if CYTHON_COMPILING_IN_CPYTHON
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_5 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1); 
      __pyx_t_4 = PyTuple_GET_ITEM(sequence, 2); 
    } else {
      __pyx_t_5 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_3 = PyList_GET_ITEM(sequence, 1); 
      __pyx_t_4 = PyList_GET_ITEM(sequence, 2); 
    }
    __Pyx_INCREF(__pyx_t_5);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_4);
    #else
    __pyx_t_5 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 233; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_3 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 233; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 233; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    #endif
  } else {
    Py_ssize_t index = -1;
    __pyx_t_7 = PyObject_GetIter(__pyx_v_vertexinfo); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 233; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = Py_TYPE(__pyx_t_7)->tp_iternext;
    index = 0; __pyx_t_5 = __pyx_t_9(__pyx_t_7); if (unlikely(!__pyx_t_5)) goto __pyx_L5_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_5);
    index = 1; __pyx_t_3 = __pyx_t_9(__pyx_t_7); if (unlikely(!__pyx_t_3)) goto __pyx_L5_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_3);
    index = 2; __pyx_t_4 = __pyx_t_9(__pyx_t_7); if (unlikely(!__pyx_t_4)) goto __pyx_L5_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_4);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_9(__pyx_t_7), 3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 233; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_9 = NULL;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    goto __pyx_L6_unpacking_done;
    __pyx_L5_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_9 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 233; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_L6_unpacking_done:;
  }
  __pyx_t_14 = __Pyx_PyInt_As_unsigned_int(__pyx_t_3); if (unlikely((__pyx_t_14 == (unsigned int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 233; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_15 = __Pyx_PyInt_As_unsigned_int(__pyx_t_4); if (unlikely((__pyx_t_15 == (unsigned int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 233; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_cnts_block = __pyx_t_5;
  __pyx_t_5 = 0;
  __pyx_v_idx_debug = __pyx_t_14;
  __pyx_v_cnt_pick = __pyx_t_15;

  /* "_gldraw.pyx":236
 * 
 *     cdef unsigned int idx_block, idx, i, j #px+
 *     idx_block = 0             # <<<<<<<<<<<<<<
 * 
 *     for idx in range(cube.number_blocks):
 */
  __pyx_v_idx_block = 0;

  /* "_gldraw.pyx":238
 *     idx_block = 0
 * 
 *     for idx in range(cube.number_blocks):             # <<<<<<<<<<<<<<
 *         cube.blocks[idx].idx_triangles = idx_block
 *         cube.blocks[idx].cnt_triangles = cnts_block[idx]
 */
  __pyx_t_15 = __pyx_v_7_gldraw_cube.number_blocks;
  for (__pyx_t_14 = 0; __pyx_t_14 < __pyx_t_15; __pyx_t_14+=1) {
    __pyx_v_idx = __pyx_t_14;

    /* "_gldraw.pyx":239
 * 
 *     for idx in range(cube.number_blocks):
 *         cube.blocks[idx].idx_triangles = idx_block             # <<<<<<<<<<<<<<
 *         cube.blocks[idx].cnt_triangles = cnts_block[idx]
 *         idx_block += cnts_block[idx]
 */
    (__pyx_v_7_gldraw_cube.blocks[__pyx_v_idx]).idx_triangles = __pyx_v_idx_block;

    /* "_gldraw.pyx":240
 *     for idx in range(cube.number_blocks):
 *         cube.blocks[idx].idx_triangles = idx_block
 *         cube.blocks[idx].cnt_triangles = cnts_block[idx]             # <<<<<<<<<<<<<<
 *         idx_block += cnts_block[idx]
 * 
 */
    __pyx_t_4 = __Pyx_GetItemInt(__pyx_v_cnts_block, __pyx_v_idx, unsigned int, 0, __Pyx_PyInt_From_unsigned_int, 0, 0, 1); if (unlikely(__pyx_t_4 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 240; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_16 = __Pyx_PyInt_As_int(__pyx_t_4); if (unlikely((__pyx_t_16 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 240; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    (__pyx_v_7_gldraw_cube.blocks[__pyx_v_idx]).cnt_triangles = __pyx_t_16;

    /* "_gldraw.pyx":241
 *         cube.blocks[idx].idx_triangles = idx_block
 *         cube.blocks[idx].cnt_triangles = cnts_block[idx]
 *         idx_block += cnts_block[idx]             # <<<<<<<<<<<<<<
 * 
 *     cube.cnt_pick = cnt_pick
 */
    __pyx_t_4 = __Pyx_PyInt_From_unsigned_int(__pyx_v_idx_block); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 241; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_cnts_block, __pyx_v_idx, unsigned int, 0, __Pyx_PyInt_From_unsigned_int, 0, 0, 1); if (unlikely(__pyx_t_3 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 241; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = PyNumber_InPlaceAdd(__pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 241; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_17 = __Pyx_PyInt_As_unsigned_int(__pyx_t_5); if (unlikely((__pyx_t_17 == (unsigned int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 241; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_idx_block = __pyx_t_17;
  }

  /* "_gldraw.pyx":243
 *         idx_block += cnts_block[idx]
 * 
 *     cube.cnt_pick = cnt_pick             # <<<<<<<<<<<<<<
 *     cube.idx_debug = idx_debug
 * 
 */
  __pyx_v_7_gldraw_cube.cnt_pick = __pyx_v_cnt_pick;

  /* "_gldraw.pyx":244
 * 
 *     cube.cnt_pick = cnt_pick
 *     cube.idx_debug = idx_debug             # <<<<<<<<<<<<<<
 * 
 *     glBindBuffer(GL_ARRAY_BUFFER, cube.glbuffer)
 */
  __pyx_v_7_gldraw_cube.idx_debug = __pyx_v_idx_debug;

  /* "_gldraw.pyx":246
 *     cube.idx_debug = idx_debug
 * 
 *     glBindBuffer(GL_ARRAY_BUFFER, cube.glbuffer)             # <<<<<<<<<<<<<<
 *     glBufferData(GL_ARRAY_BUFFER, len(vertexdata), <char*>vertexdata, GL_STATIC_DRAW) #px/
 *     #glBufferData(GL_ARRAY_BUFFER, len(vertexdata), vertexdata, GL_STATIC_DRAW)
 */
  glBindBuffer(GL_ARRAY_BUFFER, __pyx_v_7_gldraw_cube.glbuffer);

  /* "_gldraw.pyx":247
 * 
 *     glBindBuffer(GL_ARRAY_BUFFER, cube.glbuffer)
 *     glBufferData(GL_ARRAY_BUFFER, len(vertexdata), <char*>vertexdata, GL_STATIC_DRAW) #px/             # <<<<<<<<<<<<<<
 *     #glBufferData(GL_ARRAY_BUFFER, len(vertexdata), vertexdata, GL_STATIC_DRAW)
 * 
 */
  if (unlikely(__pyx_v_vertexdata == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 247; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_6 = PyBytes_GET_SIZE(__pyx_v_vertexdata); if (unlikely(__pyx_t_6 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 247; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_18 = __Pyx_PyObject_AsString(__pyx_v_vertexdata); if (unlikely((!__pyx_t_18) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 247; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  glBufferData(GL_ARRAY_BUFFER, __pyx_t_6, ((char *)__pyx_t_18), GL_STATIC_DRAW);

  /* "_gldraw.pyx":250
 *     #glBufferData(GL_ARRAY_BUFFER, len(vertexdata), vertexdata, GL_STATIC_DRAW)
 * 
 *     _gl_set_pointer(ATTRIB_LOCATION,   3, GL_FLOAT, GL_FALSE, 0)             # <<<<<<<<<<<<<<
 *     _gl_set_pointer(ATTRIB_LOCATION+1, 3, GL_FLOAT, GL_FALSE, normalpointer)
 *     _gl_set_pointer(ATTRIB_LOCATION+2, 3, GL_UNSIGNED_BYTE, GL_TRUE, colorpointer)
 */
  __pyx_f_7_gldraw__gl_set_pointer(__pyx_e_7_gldraw_ATTRIB_LOCATION, 3, GL_FLOAT, GL_FALSE, 0);

  /* "_gldraw.pyx":251
 * 
 *     _gl_set_pointer(ATTRIB_LOCATION,   3, GL_FLOAT, GL_FALSE, 0)
 *     _gl_set_pointer(ATTRIB_LOCATION+1, 3, GL_FLOAT, GL_FALSE, normalpointer)             # <<<<<<<<<<<<<<
 *     _gl_set_pointer(ATTRIB_LOCATION+2, 3, GL_UNSIGNED_BYTE, GL_TRUE, colorpointer)
 *     _gl_set_pointer(ATTRIB_LOCATION+3, 2, GL_FLOAT, GL_FALSE, texpospointer)
 */
  __pyx_f_7_gldraw__gl_set_pointer((__pyx_e_7_gldraw_ATTRIB_LOCATION + 1), 3, GL_FLOAT, GL_FALSE, __pyx_v_normalpointer);

  /* "_gldraw.pyx":252
 *     _gl_set_pointer(ATTRIB_LOCATION,   3, GL_FLOAT, GL_FALSE, 0)
 *     _gl_set_pointer(ATTRIB_LOCATION+1, 3, GL_FLOAT, GL_FALSE, normalpointer)
 *     _gl_set_pointer(ATTRIB_LOCATION+2, 3, GL_UNSIGNED_BYTE, GL_TRUE, colorpointer)             # <<<<<<<<<<<<<<
 *     _gl_set_pointer(ATTRIB_LOCATION+3, 2, GL_FLOAT, GL_FALSE, texpospointer)
 *     _gl_set_pointer(ATTRIB_LOCATION+4, 3, GL_FLOAT, GL_FALSE, barycpointer)
 */
  __pyx_f_7_gldraw__gl_set_pointer((__pyx_e_7_gldraw_ATTRIB_LOCATION + 2), 3, GL_UNSIGNED_BYTE, GL_TRUE, __pyx_v_colorpointer);

  /* "_gldraw.pyx":253
 *     _gl_set_pointer(ATTRIB_LOCATION+1, 3, GL_FLOAT, GL_FALSE, normalpointer)
 *     _gl_set_pointer(ATTRIB_LOCATION+2, 3, GL_UNSIGNED_BYTE, GL_TRUE, colorpointer)
 *     _gl_set_pointer(ATTRIB_LOCATION+3, 2, GL_FLOAT, GL_FALSE, texpospointer)             # <<<<<<<<<<<<<<
 *     _gl_set_pointer(ATTRIB_LOCATION+4, 3, GL_FLOAT, GL_FALSE, barycpointer)
 *     _gl_set_pointer(PICKATTRIB_LOCATION, 3, GL_FLOAT, GL_FALSE, pickvertpointer)
 */
  __pyx_f_7_gldraw__gl_set_pointer((__pyx_e_7_gldraw_ATTRIB_LOCATION + 3), 2, GL_FLOAT, GL_FALSE, __pyx_v_texpospointer);

  /* "_gldraw.pyx":254
 *     _gl_set_pointer(ATTRIB_LOCATION+2, 3, GL_UNSIGNED_BYTE, GL_TRUE, colorpointer)
 *     _gl_set_pointer(ATTRIB_LOCATION+3, 2, GL_FLOAT, GL_FALSE, texpospointer)
 *     _gl_set_pointer(ATTRIB_LOCATION+4, 3, GL_FLOAT, GL_FALSE, barycpointer)             # <<<<<<<<<<<<<<
 *     _gl_set_pointer(PICKATTRIB_LOCATION, 3, GL_FLOAT, GL_FALSE, pickvertpointer)
 *     _gl_set_pointer(PICKATTRIB_LOCATION+1, 3, GL_UNSIGNED_BYTE, GL_TRUE, pickcolorpointer)
 */
  __pyx_f_7_gldraw__gl_set_pointer((__pyx_e_7_gldraw_ATTRIB_LOCATION + 4), 3, GL_FLOAT, GL_FALSE, __pyx_v_barycpointer);

  /* "_gldraw.pyx":255
 *     _gl_set_pointer(ATTRIB_LOCATION+3, 2, GL_FLOAT, GL_FALSE, texpospointer)
 *     _gl_set_pointer(ATTRIB_LOCATION+4, 3, GL_FLOAT, GL_FALSE, barycpointer)
 *     _gl_set_pointer(PICKATTRIB_LOCATION, 3, GL_FLOAT, GL_FALSE, pickvertpointer)             # <<<<<<<<<<<<<<
 *     _gl_set_pointer(PICKATTRIB_LOCATION+1, 3, GL_UNSIGNED_BYTE, GL_TRUE, pickcolorpointer)
 * 
 */
  __pyx_t_13 = __Pyx_PyInt_As_long(__pyx_v_pickvertpointer); if (unlikely((__pyx_t_13 == (long)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 255; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_f_7_gldraw__gl_set_pointer(__pyx_e_7_gldraw_PICKATTRIB_LOCATION, 3, GL_FLOAT, GL_FALSE, __pyx_t_13);

  /* "_gldraw.pyx":256
 *     _gl_set_pointer(ATTRIB_LOCATION+4, 3, GL_FLOAT, GL_FALSE, barycpointer)
 *     _gl_set_pointer(PICKATTRIB_LOCATION, 3, GL_FLOAT, GL_FALSE, pickvertpointer)
 *     _gl_set_pointer(PICKATTRIB_LOCATION+1, 3, GL_UNSIGNED_BYTE, GL_TRUE, pickcolorpointer)             # <<<<<<<<<<<<<<
 * 
 *     assert len(transformations) <= MAX_TRANSFORMATIONS, '{} transformations, hardcoded limit is {}'.format(
 */
  __pyx_t_13 = __Pyx_PyInt_As_long(__pyx_v_pickcolorpointer); if (unlikely((__pyx_t_13 == (long)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 256; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_f_7_gldraw__gl_set_pointer((__pyx_e_7_gldraw_PICKATTRIB_LOCATION + 1), 3, GL_UNSIGNED_BYTE, GL_TRUE, __pyx_t_13);

  /* "_gldraw.pyx":258
 *     _gl_set_pointer(PICKATTRIB_LOCATION+1, 3, GL_UNSIGNED_BYTE, GL_TRUE, pickcolorpointer)
 * 
 *     assert len(transformations) <= MAX_TRANSFORMATIONS, '{} transformations, hardcoded limit is {}'.format(             # <<<<<<<<<<<<<<
 *                                                         len(transformations), MAX_TRANSFORMATIONS)
 *     for idx in range(len(transformations)):
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    __pyx_t_6 = PyObject_Length(__pyx_v_transformations); if (unlikely(__pyx_t_6 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 258; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (unlikely(!((__pyx_t_6 <= __pyx_e_7_gldraw_MAX_TRANSFORMATIONS) != 0))) {
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_transformations_hardcoded_limit, __pyx_n_s_format); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 258; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);

      /* "_gldraw.pyx":259
 * 
 *     assert len(transformations) <= MAX_TRANSFORMATIONS, '{} transformations, hardcoded limit is {}'.format(
 *                                                         len(transformations), MAX_TRANSFORMATIONS)             # <<<<<<<<<<<<<<
 *     for idx in range(len(transformations)):
 *         for i in range(4):
 */
      __pyx_t_19 = PyObject_Length(__pyx_v_transformations); if (unlikely(__pyx_t_19 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_t_4 = PyInt_FromSsize_t(__pyx_t_19); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_e_7_gldraw_MAX_TRANSFORMATIONS); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_1 = NULL;
      __pyx_t_19 = 0;
      if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_1)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_1);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
          __pyx_t_19 = 1;
        }
      }
      __pyx_t_2 = PyTuple_New(2+__pyx_t_19); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 258; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      if (__pyx_t_1) {
        PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1); __Pyx_GIVEREF(__pyx_t_1); __pyx_t_1 = NULL;
      }
      PyTuple_SET_ITEM(__pyx_t_2, 0+__pyx_t_19, __pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_2, 1+__pyx_t_19, __pyx_t_7);
      __Pyx_GIVEREF(__pyx_t_7);
      __pyx_t_4 = 0;
      __pyx_t_7 = 0;
      __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_2, NULL); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 258; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "_gldraw.pyx":258
 *     _gl_set_pointer(PICKATTRIB_LOCATION+1, 3, GL_UNSIGNED_BYTE, GL_TRUE, pickcolorpointer)
 * 
 *     assert len(transformations) <= MAX_TRANSFORMATIONS, '{} transformations, hardcoded limit is {}'.format(             # <<<<<<<<<<<<<<
 *                                                         len(transformations), MAX_TRANSFORMATIONS)
 *     for idx in range(len(transformations)):
 */
      __pyx_t_3 = PyTuple_Pack(1, __pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 258; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      PyErr_SetObject(PyExc_AssertionError, __pyx_t_3);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 258; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  #endif

  /* "_gldraw.pyx":260
 *     assert len(transformations) <= MAX_TRANSFORMATIONS, '{} transformations, hardcoded limit is {}'.format(
 *                                                         len(transformations), MAX_TRANSFORMATIONS)
 *     for idx in range(len(transformations)):             # <<<<<<<<<<<<<<
 *         for i in range(4):
 *             for j in range(4):
 */
  __pyx_t_6 = PyObject_Length(__pyx_v_transformations); if (unlikely(__pyx_t_6 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 260; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  for (__pyx_t_15 = 0; __pyx_t_15 < __pyx_t_6; __pyx_t_15+=1) {
    __pyx_v_idx = __pyx_t_15;

    /* "_gldraw.pyx":261
 *                                                         len(transformations), MAX_TRANSFORMATIONS)
 *     for idx in range(len(transformations)):
 *         for i in range(4):             # <<<<<<<<<<<<<<
 *             for j in range(4):
 *                 cube.transformations[idx][i][j] = float(transformations[idx][i][j])
 */
    for (__pyx_t_14 = 0; __pyx_t_14 < 4; __pyx_t_14+=1) {
      __pyx_v_i = __pyx_t_14;

      /* "_gldraw.pyx":262
 *     for idx in range(len(transformations)):
 *         for i in range(4):
 *             for j in range(4):             # <<<<<<<<<<<<<<
 *                 cube.transformations[idx][i][j] = float(transformations[idx][i][j])
 * 
 */
      for (__pyx_t_17 = 0; __pyx_t_17 < 4; __pyx_t_17+=1) {
        __pyx_v_j = __pyx_t_17;

        /* "_gldraw.pyx":263
 *         for i in range(4):
 *             for j in range(4):
 *                 cube.transformations[idx][i][j] = float(transformations[idx][i][j])             # <<<<<<<<<<<<<<
 * 
 * cdef void gl_draw_cube_debug(): #pxd/
 */
        __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_transformations, __pyx_v_idx, unsigned int, 0, __Pyx_PyInt_From_unsigned_int, 0, 0, 1); if (unlikely(__pyx_t_3 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 263; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_5 = __Pyx_GetItemInt(__pyx_t_3, __pyx_v_i, unsigned int, 0, __Pyx_PyInt_From_unsigned_int, 0, 0, 1); if (unlikely(__pyx_t_5 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 263; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_GetItemInt(__pyx_t_5, __pyx_v_j, unsigned int, 0, __Pyx_PyInt_From_unsigned_int, 0, 0, 1); if (unlikely(__pyx_t_3 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 263; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_20 = __Pyx_PyObject_AsDouble(__pyx_t_3); if (unlikely(__pyx_t_20 == ((double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 263; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        (((__pyx_v_7_gldraw_cube.transformations[__pyx_v_idx])[__pyx_v_i])[__pyx_v_j]) = __pyx_t_20;
      }
    }
  }

  /* "_gldraw.pyx":223
 *     glEnableVertexAttribArray(index)
 * 
 * cpdef gl_set_data(int nblocks, bytes vertexdata, vertexpointers, vertexinfo, transformations): #px/             # <<<<<<<<<<<<<<
 * #def gl_set_data(nblocks, vertexdata, vertexpointers, vertexinfo, transformations):
 *     assert nblocks <= MAX_BLOCKS, '{} blocks, hardcoded limit is {}'.format(
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("_gldraw.gl_set_data", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_pickvertpointer);
  __Pyx_XDECREF(__pyx_v_pickcolorpointer);
  __Pyx_XDECREF(__pyx_v_cnts_block);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_7_gldraw_11gl_set_data(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_7_gldraw_11gl_set_data(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_v_nblocks;
  PyObject *__pyx_v_vertexdata = 0;
  PyObject *__pyx_v_vertexpointers = 0;
  PyObject *__pyx_v_vertexinfo = 0;
  PyObject *__pyx_v_transformations = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("gl_set_data (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_nblocks,&__pyx_n_s_vertexdata,&__pyx_n_s_vertexpointers,&__pyx_n_s_vertexinfo,&__pyx_n_s_transformations,0};
    PyObject* values[5] = {0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_nblocks)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_vertexdata)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("gl_set_data", 1, 5, 5, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 223; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_vertexpointers)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("gl_set_data", 1, 5, 5, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 223; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_vertexinfo)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("gl_set_data", 1, 5, 5, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 223; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_transformations)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("gl_set_data", 1, 5, 5, 4); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 223; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "gl_set_data") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 223; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 5) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
    }
    __pyx_v_nblocks = __Pyx_PyInt_As_int(values[0]); if (unlikely((__pyx_v_nblocks == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 223; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_vertexdata = ((PyObject*)values[1]);
    __pyx_v_vertexpointers = values[2];
    __pyx_v_vertexinfo = values[3];
    __pyx_v_transformations = values[4];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("gl_set_data", 1, 5, 5, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 223; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("_gldraw.gl_set_data", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_vertexdata), (&PyBytes_Type), 1, "vertexdata", 1))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 223; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_7_gldraw_10gl_set_data(__pyx_self, __pyx_v_nblocks, __pyx_v_vertexdata, __pyx_v_vertexpointers, __pyx_v_vertexinfo, __pyx_v_transformations);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7_gldraw_10gl_set_data(CYTHON_UNUSED PyObject *__pyx_self, int __pyx_v_nblocks, PyObject *__pyx_v_vertexdata, PyObject *__pyx_v_vertexpointers, PyObject *__pyx_v_vertexinfo, PyObject *__pyx_v_transformations) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("gl_set_data", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_7_gldraw_gl_set_data(__pyx_v_nblocks, __pyx_v_vertexdata, __pyx_v_vertexpointers, __pyx_v_vertexinfo, __pyx_v_transformations, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 223; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("_gldraw.gl_set_data", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_gldraw.pyx":265
 *                 cube.transformations[idx][i][j] = float(transformations[idx][i][j])
 * 
 * cdef void gl_draw_cube_debug(): #pxd/             # <<<<<<<<<<<<<<
 * #def gl_draw_cube_debug():
 *     cdef mat4 object_matrix #px/
 */

static void __pyx_f_7_gldraw_gl_draw_cube_debug(void) {
  __pyx_t_7_gldraw_mat4 __pyx_v_object_matrix;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("gl_draw_cube_debug", 0);

  /* "_gldraw.pyx":269
 *     cdef mat4 object_matrix #px/
 *     #object_matrix = mat4()
 *     matrix_set_identity(object_matrix)             # <<<<<<<<<<<<<<
 *     glUniformMatrix4fv(cube.object_location, 1, GL_FALSE, &object_matrix[0][0]) #px/
 *     #glUniformMatrix4fv(cube.object_location, 1, GL_FALSE, object_matrix)
 */
  __pyx_t_1 = __pyx_f_7_gldraw_matrix_set_identity(__pyx_v_object_matrix); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 269; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "_gldraw.pyx":270
 *     #object_matrix = mat4()
 *     matrix_set_identity(object_matrix)
 *     glUniformMatrix4fv(cube.object_location, 1, GL_FALSE, &object_matrix[0][0]) #px/             # <<<<<<<<<<<<<<
 *     #glUniformMatrix4fv(cube.object_location, 1, GL_FALSE, object_matrix)
 *     glDrawArrays(GL_LINES, cube.idx_debug, 6)
 */
  glUniformMatrix4fv(__pyx_v_7_gldraw_cube.object_location, 1, GL_FALSE, (&((__pyx_v_object_matrix[0])[0])));

  /* "_gldraw.pyx":272
 *     glUniformMatrix4fv(cube.object_location, 1, GL_FALSE, &object_matrix[0][0]) #px/
 *     #glUniformMatrix4fv(cube.object_location, 1, GL_FALSE, object_matrix)
 *     glDrawArrays(GL_LINES, cube.idx_debug, 6)             # <<<<<<<<<<<<<<
 * 
 * cdef void gl_draw_select_debug(GLfloat *selectdata, GLsizeiptr size, GLuint prog_hud): #pxd/
 */
  glDrawArrays(GL_LINES, __pyx_v_7_gldraw_cube.idx_debug, 6);

  /* "_gldraw.pyx":265
 *                 cube.transformations[idx][i][j] = float(transformations[idx][i][j])
 * 
 * cdef void gl_draw_cube_debug(): #pxd/             # <<<<<<<<<<<<<<
 * #def gl_draw_cube_debug():
 *     cdef mat4 object_matrix #px/
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_WriteUnraisable("_gldraw.gl_draw_cube_debug", __pyx_clineno, __pyx_lineno, __pyx_filename, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "_gldraw.pyx":274
 *     glDrawArrays(GL_LINES, cube.idx_debug, 6)
 * 
 * cdef void gl_draw_select_debug(GLfloat *selectdata, GLsizeiptr size, GLuint prog_hud): #pxd/             # <<<<<<<<<<<<<<
 * #def gl_draw_select_debug(selectdata, size, prog_hud):
 *     cdef int i, j #px+
 */

static void __pyx_f_7_gldraw_gl_draw_select_debug(__pyx_t_2gl_GLfloat *__pyx_v_selectdata, __pyx_t_2gl_GLsizeiptr __pyx_v_size, __pyx_t_2gl_GLuint __pyx_v_prog_hud) {
  __pyx_t_2gl_GLintptr __pyx_v_offset;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("gl_draw_select_debug", 0);

  /* "_gldraw.pyx":278
 *     cdef int i, j #px+
 *     cdef GLintptr offset #px+
 *     offset = (cube.idx_debug+6) * 3 * sizeof(GLfloat)             # <<<<<<<<<<<<<<
 *     glBufferSubData(GL_ARRAY_BUFFER, offset, size, &selectdata[0]) #px/
 *     #glBufferSubData(GL_ARRAY_BUFFER, offset, len(selectdata) * sizeof(GLfloat), ArrayDatatype.asArray(selectdata, GL_FLOAT))
 */
  __pyx_v_offset = (((__pyx_v_7_gldraw_cube.idx_debug + 6) * 3) * (sizeof(__pyx_t_2gl_GLfloat)));

  /* "_gldraw.pyx":279
 *     cdef GLintptr offset #px+
 *     offset = (cube.idx_debug+6) * 3 * sizeof(GLfloat)
 *     glBufferSubData(GL_ARRAY_BUFFER, offset, size, &selectdata[0]) #px/             # <<<<<<<<<<<<<<
 *     #glBufferSubData(GL_ARRAY_BUFFER, offset, len(selectdata) * sizeof(GLfloat), ArrayDatatype.asArray(selectdata, GL_FLOAT))
 * 
 */
  glBufferSubData(GL_ARRAY_BUFFER, __pyx_v_offset, __pyx_v_size, (&(__pyx_v_selectdata[0])));

  /* "_gldraw.pyx":282
 *     #glBufferSubData(GL_ARRAY_BUFFER, offset, len(selectdata) * sizeof(GLfloat), ArrayDatatype.asArray(selectdata, GL_FLOAT))
 * 
 *     glDisable(GL_DEPTH_TEST)             # <<<<<<<<<<<<<<
 *     glDrawArrays(GL_LINES, cube.idx_debug+6, 2)
 *     glUseProgram(prog_hud)
 */
  glDisable(GL_DEPTH_TEST);

  /* "_gldraw.pyx":283
 * 
 *     glDisable(GL_DEPTH_TEST)
 *     glDrawArrays(GL_LINES, cube.idx_debug+6, 2)             # <<<<<<<<<<<<<<
 *     glUseProgram(prog_hud)
 *     glDrawArrays(GL_POINTS, cube.idx_debug+6+2, 2)
 */
  glDrawArrays(GL_LINES, (__pyx_v_7_gldraw_cube.idx_debug + 6), 2);

  /* "_gldraw.pyx":284
 *     glDisable(GL_DEPTH_TEST)
 *     glDrawArrays(GL_LINES, cube.idx_debug+6, 2)
 *     glUseProgram(prog_hud)             # <<<<<<<<<<<<<<
 *     glDrawArrays(GL_POINTS, cube.idx_debug+6+2, 2)
 *     glEnable(GL_DEPTH_TEST)
 */
  glUseProgram(__pyx_v_prog_hud);

  /* "_gldraw.pyx":285
 *     glDrawArrays(GL_LINES, cube.idx_debug+6, 2)
 *     glUseProgram(prog_hud)
 *     glDrawArrays(GL_POINTS, cube.idx_debug+6+2, 2)             # <<<<<<<<<<<<<<
 *     glEnable(GL_DEPTH_TEST)
 * 
 */
  glDrawArrays(GL_POINTS, ((__pyx_v_7_gldraw_cube.idx_debug + 6) + 2), 2);

  /* "_gldraw.pyx":286
 *     glUseProgram(prog_hud)
 *     glDrawArrays(GL_POINTS, cube.idx_debug+6+2, 2)
 *     glEnable(GL_DEPTH_TEST)             # <<<<<<<<<<<<<<
 * 
 * cdef void gl_init_object_location(GLuint location): #pxd/
 */
  glEnable(GL_DEPTH_TEST);

  /* "_gldraw.pyx":274
 *     glDrawArrays(GL_LINES, cube.idx_debug, 6)
 * 
 * cdef void gl_draw_select_debug(GLfloat *selectdata, GLsizeiptr size, GLuint prog_hud): #pxd/             # <<<<<<<<<<<<<<
 * #def gl_draw_select_debug(selectdata, size, prog_hud):
 *     cdef int i, j #px+
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "_gldraw.pyx":288
 *     glEnable(GL_DEPTH_TEST)
 * 
 * cdef void gl_init_object_location(GLuint location): #pxd/             # <<<<<<<<<<<<<<
 * #def gl_init_object_location(location):
 *     cube.object_location = location
 */

static void __pyx_f_7_gldraw_gl_init_object_location(__pyx_t_2gl_GLuint __pyx_v_location) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("gl_init_object_location", 0);

  /* "_gldraw.pyx":290
 * cdef void gl_init_object_location(GLuint location): #pxd/
 * #def gl_init_object_location(location):
 *     cube.object_location = location             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_7_gldraw_cube.object_location = __pyx_v_location;

  /* "_gldraw.pyx":288
 *     glEnable(GL_DEPTH_TEST)
 * 
 * cdef void gl_init_object_location(GLuint location): #pxd/             # <<<<<<<<<<<<<<
 * #def gl_init_object_location(location):
 *     cube.object_location = location
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static PyMethodDef __pyx_methods[] = {
  {"gl_set_atlas_texture", (PyCFunction)__pyx_pw_7_gldraw_5gl_set_atlas_texture, METH_VARARGS|METH_KEYWORDS, 0},
  {"set_animation_start", (PyCFunction)__pyx_pw_7_gldraw_7set_animation_start, METH_VARARGS|METH_KEYWORDS, 0},
  {"set_animation_next", (PyCFunction)__pyx_pw_7_gldraw_9set_animation_next, METH_O, 0},
  {"gl_set_data", (PyCFunction)__pyx_pw_7_gldraw_11gl_set_data, METH_VARARGS|METH_KEYWORDS, 0},
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
static struct PyModuleDef __pyx_moduledef = {
  #if PY_VERSION_HEX < 0x03020000
    { PyObject_HEAD_INIT(NULL) NULL, 0, NULL },
  #else
    PyModuleDef_HEAD_INIT,
  #endif
    "_gldraw",
    0, /* m_doc */
    -1, /* m_size */
    __pyx_methods /* m_methods */,
    NULL, /* m_reload */
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_n_u_ATTRIB_LOCATION, __pyx_k_ATTRIB_LOCATION, sizeof(__pyx_k_ATTRIB_LOCATION), 0, 1, 0, 1},
  {&__pyx_n_s_DEBUG, __pyx_k_DEBUG, sizeof(__pyx_k_DEBUG), 0, 0, 1, 1},
  {&__pyx_kp_u_Importing_module, __pyx_k_Importing_module, sizeof(__pyx_k_Importing_module), 0, 1, 0, 0},
  {&__pyx_n_u_PICKATTRIB_LOCATION, __pyx_k_PICKATTRIB_LOCATION, sizeof(__pyx_k_PICKATTRIB_LOCATION), 0, 1, 0, 1},
  {&__pyx_n_s_all, __pyx_k_all, sizeof(__pyx_k_all), 0, 0, 1, 1},
  {&__pyx_n_s_angle_max, __pyx_k_angle_max, sizeof(__pyx_k_angle_max), 0, 0, 1, 1},
  {&__pyx_n_s_axisx, __pyx_k_axisx, sizeof(__pyx_k_axisx), 0, 0, 1, 1},
  {&__pyx_n_s_axisy, __pyx_k_axisy, sizeof(__pyx_k_axisy), 0, 0, 1, 1},
  {&__pyx_n_s_axisz, __pyx_k_axisz, sizeof(__pyx_k_axisz), 0, 0, 1, 1},
  {&__pyx_n_s_b, __pyx_k_b, sizeof(__pyx_k_b), 0, 0, 1, 1},
  {&__pyx_n_s_blocks, __pyx_k_blocks, sizeof(__pyx_k_blocks), 0, 0, 1, 1},
  {&__pyx_kp_u_blocks_hardcoded_limit_is, __pyx_k_blocks_hardcoded_limit_is, sizeof(__pyx_k_blocks_hardcoded_limit_is), 0, 1, 0, 0},
  {&__pyx_n_s_compiled, __pyx_k_compiled, sizeof(__pyx_k_compiled), 0, 0, 1, 1},
  {&__pyx_kp_u_compiled_2, __pyx_k_compiled_2, sizeof(__pyx_k_compiled_2), 0, 1, 0, 0},
  {&__pyx_n_s_data, __pyx_k_data, sizeof(__pyx_k_data), 0, 0, 1, 1},
  {&__pyx_n_s_debug, __pyx_k_debug, sizeof(__pyx_k_debug), 0, 0, 1, 1},
  {&__pyx_n_s_enumerate, __pyx_k_enumerate, sizeof(__pyx_k_enumerate), 0, 0, 1, 1},
  {&__pyx_n_s_format, __pyx_k_format, sizeof(__pyx_k_format), 0, 0, 1, 1},
  {&__pyx_kp_u_from_package, __pyx_k_from_package, sizeof(__pyx_k_from_package), 0, 1, 0, 0},
  {&__pyx_n_u_gl_delete_buffers, __pyx_k_gl_delete_buffers, sizeof(__pyx_k_gl_delete_buffers), 0, 1, 0, 1},
  {&__pyx_n_u_gl_draw_cube, __pyx_k_gl_draw_cube, sizeof(__pyx_k_gl_draw_cube), 0, 1, 0, 1},
  {&__pyx_n_u_gl_draw_cube_debug, __pyx_k_gl_draw_cube_debug, sizeof(__pyx_k_gl_draw_cube_debug), 0, 1, 0, 1},
  {&__pyx_n_u_gl_draw_select_debug, __pyx_k_gl_draw_select_debug, sizeof(__pyx_k_gl_draw_select_debug), 0, 1, 0, 1},
  {&__pyx_n_u_gl_init_buffers, __pyx_k_gl_init_buffers, sizeof(__pyx_k_gl_init_buffers), 0, 1, 0, 1},
  {&__pyx_n_u_gl_init_object_location, __pyx_k_gl_init_object_location, sizeof(__pyx_k_gl_init_object_location), 0, 1, 0, 1},
  {&__pyx_n_u_gl_pick_cube, __pyx_k_gl_pick_cube, sizeof(__pyx_k_gl_pick_cube), 0, 1, 0, 1},
  {&__pyx_n_s_gldraw, __pyx_k_gldraw, sizeof(__pyx_k_gldraw), 0, 0, 1, 1},
  {&__pyx_n_s_h, __pyx_k_h, sizeof(__pyx_k_h), 0, 0, 1, 1},
  {&__pyx_kp_s_tmp, __pyx_k_tmp, sizeof(__pyx_k_tmp), 0, 0, 1, 0},
  {&__pyx_n_s_i, __pyx_k_i, sizeof(__pyx_k_i), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_init_module, __pyx_k_init_module, sizeof(__pyx_k_init_module), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_u_mat4, __pyx_k_mat4, sizeof(__pyx_k_mat4), 0, 1, 0, 1},
  {&__pyx_n_u_matrix_set_identity, __pyx_k_matrix_set_identity, sizeof(__pyx_k_matrix_set_identity), 0, 1, 0, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_nblocks, __pyx_k_nblocks, sizeof(__pyx_k_nblocks), 0, 0, 1, 1},
  {&__pyx_n_s_package, __pyx_k_package, sizeof(__pyx_k_package), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_set_transformations, __pyx_k_set_transformations, sizeof(__pyx_k_set_transformations), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_transformations, __pyx_k_transformations, sizeof(__pyx_k_transformations), 0, 0, 1, 1},
  {&__pyx_kp_u_transformations_hardcoded_limit, __pyx_k_transformations_hardcoded_limit, sizeof(__pyx_k_transformations_hardcoded_limit), 0, 1, 0, 0},
  {&__pyx_n_s_vertexdata, __pyx_k_vertexdata, sizeof(__pyx_k_vertexdata), 0, 0, 1, 1},
  {&__pyx_n_s_vertexinfo, __pyx_k_vertexinfo, sizeof(__pyx_k_vertexinfo), 0, 0, 1, 1},
  {&__pyx_n_s_vertexpointers, __pyx_k_vertexpointers, sizeof(__pyx_k_vertexpointers), 0, 0, 1, 1},
  {&__pyx_n_s_w, __pyx_k_w, sizeof(__pyx_k_w), 0, 0, 1, 1},
  {&__pyx_n_s_x, __pyx_k_x, sizeof(__pyx_k_x), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_enumerate = __Pyx_GetBuiltinName(__pyx_n_s_enumerate); if (!__pyx_builtin_enumerate) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 179; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "_gldraw.pyx":114
 * 
 * 
 * def init_module():             # <<<<<<<<<<<<<<
 *     cdef int i #px+
 * 
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_n_s_i); if (unlikely(!__pyx_tuple_)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 114; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);
  __pyx_codeobj__2 = (PyObject*)__Pyx_PyCode_New(0, 0, 1, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple_, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_tmp, __pyx_n_s_init_module, 114, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 114; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "_gldraw.pyx":125
 *     animation.angle = animation.angle_max = 0
 * 
 * def set_transformations(blocks):             # <<<<<<<<<<<<<<
 *     cdef unsigned int i, b #px+
 *     for i, b in enumerate(blocks):
 */
  __pyx_tuple__3 = PyTuple_Pack(3, __pyx_n_s_blocks, __pyx_n_s_i, __pyx_n_s_b); if (unlikely(!__pyx_tuple__3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);
  __pyx_codeobj__4 = (PyObject*)__Pyx_PyCode_New(1, 0, 3, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__3, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_tmp, __pyx_n_s_set_transformations, 125, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC init_gldraw(void); /*proto*/
PyMODINIT_FUNC init_gldraw(void)
#else
PyMODINIT_FUNC PyInit__gldraw(void); /*proto*/
PyMODINIT_FUNC PyInit__gldraw(void)
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  Py_ssize_t __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit__gldraw(void)", 0);
  if ( __Pyx_check_binary_version() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #ifdef __Pyx_CyFunction_USED
  if (__Pyx_CyFunction_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("_gldraw", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  /*--- Initialize various global constants etc. ---*/
  if (unlikely(__Pyx_InitGlobals() < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  if (__pyx_module_is_main__gldraw) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (!PyDict_GetItemString(modules, "_gldraw")) {
      if (unlikely(PyDict_SetItemString(modules, "_gldraw", __pyx_m) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (unlikely(__Pyx_InitCachedBuiltins() < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Constants init code ---*/
  if (unlikely(__Pyx_InitCachedConstants() < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Global init code ---*/
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  if (__Pyx_ExportFunction("matrix_set_identity", (void (*)(void))__pyx_f_7_gldraw_matrix_set_identity, "PyObject *(__pyx_t_7_gldraw_vec4 *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("gl_draw_cube", (void (*)(void))__pyx_f_7_gldraw_gl_draw_cube, "void (void)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("gl_pick_cube", (void (*)(void))__pyx_f_7_gldraw_gl_pick_cube, "void (void)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("gl_init_buffers", (void (*)(void))__pyx_f_7_gldraw_gl_init_buffers, "void (void)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("gl_delete_buffers", (void (*)(void))__pyx_f_7_gldraw_gl_delete_buffers, "void (void)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("gl_draw_cube_debug", (void (*)(void))__pyx_f_7_gldraw_gl_draw_cube_debug, "void (void)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("gl_draw_select_debug", (void (*)(void))__pyx_f_7_gldraw_gl_draw_select_debug, "void (__pyx_t_2gl_GLfloat *, __pyx_t_2gl_GLsizeiptr, __pyx_t_2gl_GLuint)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("gl_init_object_location", (void (*)(void))__pyx_f_7_gldraw_gl_init_object_location, "void (__pyx_t_2gl_GLuint)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Type init code ---*/
  /*--- Type import code ---*/
  /*--- Variable import code ---*/
  /*--- Function import code ---*/
  /*--- Execution code ---*/

  /* "_gldraw.pyx":25
 * # This line makes cython happy
 * global __name__, __package__    # pylint: disable=W0604
 * __compiled = True #px/             # <<<<<<<<<<<<<<
 * #__compiled = False
 * 
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_compiled, Py_True) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "_gldraw.pyx":28
 * #__compiled = False
 * 
 * __all__ = ['ATTRIB_LOCATION', 'PICKATTRIB_LOCATION',             # <<<<<<<<<<<<<<
 *            'matrix_set_identity', 'mat4',
 *            'gl_draw_cube', 'gl_pick_cube', 'gl_init_buffers', 'gl_delete_buffers',
 */
  __pyx_t_1 = PyList_New(11); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 28; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_u_ATTRIB_LOCATION);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_u_ATTRIB_LOCATION);
  __Pyx_GIVEREF(__pyx_n_u_ATTRIB_LOCATION);
  __Pyx_INCREF(__pyx_n_u_PICKATTRIB_LOCATION);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_u_PICKATTRIB_LOCATION);
  __Pyx_GIVEREF(__pyx_n_u_PICKATTRIB_LOCATION);
  __Pyx_INCREF(__pyx_n_u_matrix_set_identity);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_n_u_matrix_set_identity);
  __Pyx_GIVEREF(__pyx_n_u_matrix_set_identity);
  __Pyx_INCREF(__pyx_n_u_mat4);
  PyList_SET_ITEM(__pyx_t_1, 3, __pyx_n_u_mat4);
  __Pyx_GIVEREF(__pyx_n_u_mat4);
  __Pyx_INCREF(__pyx_n_u_gl_draw_cube);
  PyList_SET_ITEM(__pyx_t_1, 4, __pyx_n_u_gl_draw_cube);
  __Pyx_GIVEREF(__pyx_n_u_gl_draw_cube);
  __Pyx_INCREF(__pyx_n_u_gl_pick_cube);
  PyList_SET_ITEM(__pyx_t_1, 5, __pyx_n_u_gl_pick_cube);
  __Pyx_GIVEREF(__pyx_n_u_gl_pick_cube);
  __Pyx_INCREF(__pyx_n_u_gl_init_buffers);
  PyList_SET_ITEM(__pyx_t_1, 6, __pyx_n_u_gl_init_buffers);
  __Pyx_GIVEREF(__pyx_n_u_gl_init_buffers);
  __Pyx_INCREF(__pyx_n_u_gl_delete_buffers);
  PyList_SET_ITEM(__pyx_t_1, 7, __pyx_n_u_gl_delete_buffers);
  __Pyx_GIVEREF(__pyx_n_u_gl_delete_buffers);
  __Pyx_INCREF(__pyx_n_u_gl_draw_cube_debug);
  PyList_SET_ITEM(__pyx_t_1, 8, __pyx_n_u_gl_draw_cube_debug);
  __Pyx_GIVEREF(__pyx_n_u_gl_draw_cube_debug);
  __Pyx_INCREF(__pyx_n_u_gl_draw_select_debug);
  PyList_SET_ITEM(__pyx_t_1, 9, __pyx_n_u_gl_draw_select_debug);
  __Pyx_GIVEREF(__pyx_n_u_gl_draw_select_debug);
  __Pyx_INCREF(__pyx_n_u_gl_init_object_location);
  PyList_SET_ITEM(__pyx_t_1, 10, __pyx_n_u_gl_init_object_location);
  __Pyx_GIVEREF(__pyx_n_u_gl_init_object_location);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_all, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 28; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "_gldraw.pyx":43
 * ##px.
 * 
 * from .debug import debug, DEBUG             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 43; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_debug);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_debug);
  __Pyx_GIVEREF(__pyx_n_s_debug);
  __Pyx_INCREF(__pyx_n_s_DEBUG);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_s_DEBUG);
  __Pyx_GIVEREF(__pyx_n_s_DEBUG);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_debug, __pyx_t_1, 1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 43; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_debug); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 43; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_debug, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 43; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_DEBUG); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 43; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DEBUG, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 43; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "_gldraw.pyx":46
 * 
 * 
 * if DEBUG:             # <<<<<<<<<<<<<<
 *     debug('Importing module:', __name__)
 *     debug('  from package:', __package__)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_DEBUG); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 46; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 46; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {

    /* "_gldraw.pyx":47
 * 
 * if DEBUG:
 *     debug('Importing module:', __name__)             # <<<<<<<<<<<<<<
 *     debug('  from package:', __package__)
 *     debug('  compiled:', __compiled)
 */
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_debug); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 47; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_name); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 47; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    __pyx_t_6 = 0;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
        __pyx_t_6 = 1;
      }
    }
    __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 47; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_5) {
      PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __Pyx_GIVEREF(__pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_INCREF(__pyx_kp_u_Importing_module);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_kp_u_Importing_module);
    __Pyx_GIVEREF(__pyx_kp_u_Importing_module);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_7, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 47; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "_gldraw.pyx":48
 * if DEBUG:
 *     debug('Importing module:', __name__)
 *     debug('  from package:', __package__)             # <<<<<<<<<<<<<<
 *     debug('  compiled:', __compiled)
 * 
 */
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_debug); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 48; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_package); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 48; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_4 = NULL;
    __pyx_t_6 = 0;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
        __pyx_t_6 = 1;
      }
    }
    __pyx_t_5 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 48; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_4) {
      PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __Pyx_GIVEREF(__pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_INCREF(__pyx_kp_u_from_package);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_6, __pyx_kp_u_from_package);
    __Pyx_GIVEREF(__pyx_kp_u_from_package);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_6, __pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_7);
    __pyx_t_7 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 48; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "_gldraw.pyx":49
 *     debug('Importing module:', __name__)
 *     debug('  from package:', __package__)
 *     debug('  compiled:', __compiled)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_debug); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 49; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_compiled); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 49; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_7 = NULL;
    __pyx_t_6 = 0;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
        __pyx_t_6 = 1;
      }
    }
    __pyx_t_4 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 49; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    if (__pyx_t_7) {
      PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_7); __Pyx_GIVEREF(__pyx_t_7); __pyx_t_7 = NULL;
    }
    __Pyx_INCREF(__pyx_kp_u_compiled_2);
    PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_6, __pyx_kp_u_compiled_2);
    __Pyx_GIVEREF(__pyx_kp_u_compiled_2);
    PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_6, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_5);
    __pyx_t_5 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_4, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 49; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    goto __pyx_L2;
  }
  __pyx_L2:;

  /* "_gldraw.pyx":114
 * 
 * 
 * def init_module():             # <<<<<<<<<<<<<<
 *     cdef int i #px+
 * 
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_7_gldraw_1init_module, NULL, __pyx_n_s_gldraw); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 114; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_init_module, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 114; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "_gldraw.pyx":125
 *     animation.angle = animation.angle_max = 0
 * 
 * def set_transformations(blocks):             # <<<<<<<<<<<<<<
 *     cdef unsigned int i, b #px+
 *     for i, b in enumerate(blocks):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_7_gldraw_3set_transformations, NULL, __pyx_n_s_gldraw); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_set_transformations, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "_gldraw.pyx":1
 * #-*- coding:utf-8 -*-             # <<<<<<<<<<<<<<
 * # cython: profile=False
 * 
 */
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init _gldraw", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init _gldraw");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if PY_MAJOR_VERSION < 3
  return;
  #else
  return __pyx_m;
  #endif
}

/* Runtime support code */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

static void __Pyx_RaiseArgumentTypeInvalid(const char* name, PyObject *obj, PyTypeObject *type) {
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
}
static CYTHON_INLINE int __Pyx_ArgTypeTest(PyObject *obj, PyTypeObject *type, int none_allowed,
    const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (none_allowed && obj == Py_None) return 1;
    else if (exact) {
        if (likely(Py_TYPE(obj) == type)) return 1;
        #if PY_MAJOR_VERSION == 2
        else if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(PyObject_TypeCheck(obj, type))) return 1;
    }
    __Pyx_RaiseArgumentTypeInvalid(name, obj, type);
    return 0;
}

static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyThreadState *tstate = PyThreadState_GET();
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(exc_type == PyExc_StopIteration) || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration)) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    } else {
        return __Pyx_IterFinish();
    }
    return 0;
}

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck) {
#if CYTHON_COMPILING_IN_CPYTHON
    if (wraparound & unlikely(i < 0)) i += PyList_GET_SIZE(o);
    if ((!boundscheck) || likely((0 <= i) & (i < PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck) {
#if CYTHON_COMPILING_IN_CPYTHON
    if (wraparound & unlikely(i < 0)) i += PyTuple_GET_SIZE(o);
    if ((!boundscheck) || likely((0 <= i) & (i < PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck) {
#if CYTHON_COMPILING_IN_CPYTHON
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely((n >= 0) & (n < PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely((n >= 0) & (n < PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (PyErr_ExceptionMatches(PyExc_OverflowError))
                        PyErr_Clear();
                    else
                        return NULL;
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

static double __Pyx__PyObject_AsDouble(PyObject* obj) {
    PyObject* float_value;
#if CYTHON_COMPILING_IN_PYPY
    float_value = PyNumber_Float(obj);
#else
    PyNumberMethods *nb = Py_TYPE(obj)->tp_as_number;
    if (likely(nb) && likely(nb->nb_float)) {
        float_value = nb->nb_float(obj);
        if (likely(float_value) && unlikely(!PyFloat_Check(float_value))) {
            PyErr_Format(PyExc_TypeError,
                "__float__ returned non-float (type %.200s)",
                Py_TYPE(float_value)->tp_name);
            Py_DECREF(float_value);
            goto bad;
        }
    } else if (PyUnicode_CheckExact(obj) || PyBytes_CheckExact(obj)) {
#if PY_MAJOR_VERSION >= 3
        float_value = PyFloat_FromString(obj);
#else
        float_value = PyFloat_FromString(obj, 0);
#endif
    } else {
        PyObject* args = PyTuple_New(1);
        if (unlikely(!args)) goto bad;
        PyTuple_SET_ITEM(args, 0, obj);
        float_value = PyObject_Call((PyObject*)&PyFloat_Type, args, 0);
        PyTuple_SET_ITEM(args, 0, 0);
        Py_DECREF(args);
    }
#endif
    if (likely(float_value)) {
        double value = PyFloat_AS_DOUBLE(float_value);
        Py_DECREF(float_value);
        return value;
    }
bad:
    return (double)-1;
}

static CYTHON_INLINE void __Pyx_ErrRestore(PyObject *type, PyObject *value, PyObject *tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyThreadState *tstate = PyThreadState_GET();
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_Restore(type, value, tb);
#endif
}
static CYTHON_INLINE void __Pyx_ErrFetch(PyObject **type, PyObject **value, PyObject **tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyThreadState *tstate = PyThreadState_GET();
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(type, value, tb);
#endif
}

static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
}

static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name) {
    PyObject *result;
#if CYTHON_COMPILING_IN_CPYTHON
    result = PyDict_GetItem(__pyx_d, name);
    if (likely(result)) {
        Py_INCREF(result);
    } else {
#else
    result = PyObject_GetItem(__pyx_d, name);
    if (!result) {
        PyErr_Clear();
#endif
        result = __Pyx_GetBuiltinName(name);
    }
    return result;
}

static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = (start + end) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    py_code = __pyx_find_code_object(c_line ? c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        PyThreadState_GET(), /*PyThreadState *tstate,*/
        py_code,             /*PyCodeObject *code,*/
        __pyx_d,      /*PyObject *globals,*/
        0                    /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    py_frame->f_lineno = py_line;
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_VERSION_HEX < 0x03030000
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                #if PY_VERSION_HEX < 0x03030000
                PyObject *py_level = PyInt_FromLong(1);
                if (!py_level)
                    goto bad;
                module = PyObject_CallFunctionObjArgs(py_import,
                    name, global_dict, empty_dict, list, py_level, NULL);
                Py_DECREF(py_level);
                #else
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                #endif
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_VERSION_HEX < 0x03030000
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_VERSION_HEX < 0x03030000
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)       \
    {                                                                     \
        func_type value = func_value;                                     \
        if (sizeof(target_type) < sizeof(func_type)) {                    \
            if (unlikely(value != (func_type) (target_type) value)) {     \
                func_type zero = 0;                                       \
                if (is_unsigned && unlikely(value < zero))                \
                    goto raise_neg_overflow;                              \
                else                                                      \
                    goto raise_overflow;                                  \
            }                                                             \
        }                                                                 \
        return (target_type) value;                                       \
    }

#if CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
 #if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
 #endif
#endif

static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
 #if CYTHON_USE_PYLONG_INTERNALS
            switch (Py_SIZE(x)) {
                case  0: return 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, ((PyLongObject*)x)->ob_digit[0]);
            }
 #endif
#endif
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT(int, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(int) <= sizeof(unsigned long long)) {
                __PYX_VERIFY_RETURN_INT(int, unsigned long long, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
 #if CYTHON_USE_PYLONG_INTERNALS
            switch (Py_SIZE(x)) {
                case  0: return 0;
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +(((PyLongObject*)x)->ob_digit[0]));
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, -(sdigit) ((PyLongObject*)x)->ob_digit[0]);
            }
 #endif
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT(int, long, PyLong_AsLong(x))
            } else if (sizeof(int) <= sizeof(long long)) {
                __PYX_VERIFY_RETURN_INT(int, long long, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

static CYTHON_INLINE unsigned int __Pyx_PyInt_As_unsigned_int(PyObject *x) {
    const unsigned int neg_one = (unsigned int) -1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(unsigned int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(unsigned int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (unsigned int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
 #if CYTHON_USE_PYLONG_INTERNALS
            switch (Py_SIZE(x)) {
                case  0: return 0;
                case  1: __PYX_VERIFY_RETURN_INT(unsigned int, digit, ((PyLongObject*)x)->ob_digit[0]);
            }
 #endif
#endif
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
            if (sizeof(unsigned int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(unsigned int) <= sizeof(unsigned long long)) {
                __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long long, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
 #if CYTHON_USE_PYLONG_INTERNALS
            switch (Py_SIZE(x)) {
                case  0: return 0;
                case  1: __PYX_VERIFY_RETURN_INT(unsigned int,  digit, +(((PyLongObject*)x)->ob_digit[0]));
                case -1: __PYX_VERIFY_RETURN_INT(unsigned int, sdigit, -(sdigit) ((PyLongObject*)x)->ob_digit[0]);
            }
 #endif
#endif
            if (sizeof(unsigned int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT(unsigned int, long, PyLong_AsLong(x))
            } else if (sizeof(unsigned int) <= sizeof(long long)) {
                __PYX_VERIFY_RETURN_INT(unsigned int, long long, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            unsigned int val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (unsigned int) -1;
        }
    } else {
        unsigned int val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (unsigned int) -1;
        val = __Pyx_PyInt_As_unsigned_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to unsigned int");
    return (unsigned int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to unsigned int");
    return (unsigned int) -1;
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_int(unsigned int value) {
    const unsigned int neg_one = (unsigned int) -1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(unsigned int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(unsigned int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(unsigned int) <= sizeof(unsigned long long)) {
            return PyLong_FromUnsignedLongLong((unsigned long long) value);
        }
    } else {
        if (sizeof(unsigned int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(unsigned int) <= sizeof(long long)) {
            return PyLong_FromLongLong((long long) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(unsigned int),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) -1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(int) <= sizeof(unsigned long long)) {
            return PyLong_FromUnsignedLongLong((unsigned long long) value);
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(long long)) {
            return PyLong_FromLongLong((long long) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
 #if CYTHON_USE_PYLONG_INTERNALS
            switch (Py_SIZE(x)) {
                case  0: return 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, ((PyLongObject*)x)->ob_digit[0]);
            }
 #endif
#endif
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT(long, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(long) <= sizeof(unsigned long long)) {
                __PYX_VERIFY_RETURN_INT(long, unsigned long long, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
 #if CYTHON_USE_PYLONG_INTERNALS
            switch (Py_SIZE(x)) {
                case  0: return 0;
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +(((PyLongObject*)x)->ob_digit[0]));
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, -(sdigit) ((PyLongObject*)x)->ob_digit[0]);
            }
 #endif
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT(long, long, PyLong_AsLong(x))
            } else if (sizeof(long) <= sizeof(long long)) {
                __PYX_VERIFY_RETURN_INT(long, long long, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(long) <= sizeof(unsigned long long)) {
            return PyLong_FromUnsignedLongLong((unsigned long long) value);
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(long long)) {
            return PyLong_FromLongLong((long long) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

static int __Pyx_ExportFunction(const char *name, void (*f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(__pyx_m, (char *)"__pyx_capi__");
    if (!d) {
        PyErr_Clear();
        d = PyDict_New();
        if (!d)
            goto bad;
        Py_INCREF(d);
        if (PyModule_AddObject(__pyx_m, (char *)"__pyx_capi__", d) < 0)
            goto bad;
    }
    tmp.fp = f;
#if PY_VERSION_HEX >= 0x02070000
    cobj = PyCapsule_New(tmp.p, sig, 0);
#else
    cobj = PyCObject_FromVoidPtrAndDesc(tmp.p, (void *)sig, 0);
#endif
    if (!cobj)
        goto bad;
    if (PyDict_SetItemString(d, name, cobj) < 0)
        goto bad;
    Py_DECREF(cobj);
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(cobj);
    Py_XDECREF(d);
    return -1;
}

static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
static CYTHON_INLINE char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
#if PY_VERSION_HEX < 0x03030000
        char* defenc_c;
        PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
        if (!defenc) return NULL;
        defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        {
            char* end = defenc_c + PyBytes_GET_SIZE(defenc);
            char* c;
            for (c = defenc_c; c < end; c++) {
                if ((unsigned char) (*c) >= 128) {
                    PyUnicode_AsASCIIString(o);
                    return NULL;
                }
            }
        }
#endif
        *length = PyBytes_GET_SIZE(defenc);
        return defenc_c;
#else
        if (__Pyx_PyUnicode_READY(o) == -1) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        if (PyUnicode_IS_ASCII(o)) {
            *length = PyUnicode_GET_LENGTH(o);
            return PyUnicode_AsUTF8(o);
        } else {
            PyUnicode_AsASCIIString(o);
            return NULL;
        }
#else
        return PyUnicode_AsUTF8AndSize(o, length);
#endif
#endif
    } else
#endif
#if !CYTHON_COMPILING_IN_PYPY
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Int(PyObject* x) {
  PyNumberMethods *m;
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (PyInt_Check(x) || PyLong_Check(x))
#else
  if (PyLong_Check(x))
#endif
    return Py_INCREF(x), x;
  m = Py_TYPE(x)->tp_as_number;
#if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = PyNumber_Long(x);
  }
#else
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Long(x);
  }
#endif
  if (res) {
#if PY_MAJOR_VERSION < 3
    if (!PyInt_Check(res) && !PyLong_Check(res)) {
#else
    if (!PyLong_Check(res)) {
#endif
      PyErr_Format(PyExc_TypeError,
                   "__%.4s__ returned non-%.4s (type %.200s)",
                   name, name, Py_TYPE(res)->tp_name);
      Py_DECREF(res);
      return NULL;
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b)))
      return PyInt_AS_LONG(b);
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
     #if CYTHON_USE_PYLONG_INTERNALS
       switch (Py_SIZE(b)) {
       case -1: return -(sdigit)((PyLongObject*)b)->ob_digit[0];
       case  0: return 0;
       case  1: return ((PyLongObject*)b)->ob_digit[0];
       }
     #endif
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
